@@ -3,7 +3,7 @@
   bignum.c -
 
   $Author: shyouhei $
-  $Date: 2006/12/06 10:14:12 $
+  $Date: 2008-08-04 13:52:05 +0900 (Mon, 04 Aug 2008) $
   created at: Fri Jun 10 00:48:55 JST 1994
 
   Copyright (C) 1993-2003 Yukihiro Matsumoto
@@ -11,6 +11,7 @@
 **********************************************************************/
 
 #include "ruby.h"
+#include "rubysig.h"
 
 #include <math.h>
 #include <ctype.h>
@@ -111,35 +112,50 @@ rb_big_2comp(x)			/* get 2's complement */
 }
 
 static VALUE
-bignorm(x)
+bigtrunc(x)
     VALUE x;
 {
-    if (FIXNUM_P(x)) {
-      return x;
-    }
-    else if (TYPE(x) == T_BIGNUM) {
-	long len = RBIGNUM(x)->len;
-	BDIGIT *ds = BDIGITS(x);
+    long len = RBIGNUM(x)->len;
+    BDIGIT *ds = BDIGITS(x);
 
-	while (--len && !ds[len]) ;
-	RBIGNUM(x)->len = ++len;
+    if (len == 0) return x;
+    while (--len && !ds[len]);
+    RBIGNUM(x)->len = ++len;
+    return x;
+}
 
-	if (len*SIZEOF_BDIGITS <= sizeof(VALUE)) {
-	    long num = 0;
-	    while (len--) {
-		num = BIGUP(num) + ds[len];
+static VALUE
+bigfixize(VALUE x)
+{
+    long len = RBIGNUM(x)->len;
+    BDIGIT *ds = BDIGITS(x);
+
+    if (len*SIZEOF_BDIGITS <= sizeof(VALUE)) {
+	long num = 0;
+	while (len--) {
+	    num = BIGUP(num) + ds[len];
+	}
+	if (num >= 0) {
+	    if (RBIGNUM(x)->sign) {
+		if (POSFIXABLE(num)) return LONG2FIX(num);
 	    }
-	    if (num >= 0) {
-		if (RBIGNUM(x)->sign) {
-		    if (POSFIXABLE(num)) return LONG2FIX(num);
-		}
-		else if (NEGFIXABLE(-(long)num)) return LONG2FIX(-(long)num);
+	    else {
+		if (NEGFIXABLE(-(long)num)) return LONG2FIX(-(long)num);
 	    }
 	}
     }
     return x;
 }
 
+static VALUE
+bignorm(VALUE x)
+{
+    if (!FIXNUM_P(x) && TYPE(x) == T_BIGNUM) {
+	x = bigfixize(bigtrunc(x));
+    }
+    return x;
+}
+
 VALUE
 rb_big_norm(x)
     VALUE x;
@@ -633,15 +649,16 @@ rb_str2inum(str, base)
 
 const char ruby_digitmap[] = "0123456789abcdefghijklmnopqrstuvwxyz";
 VALUE
-rb_big2str(x, base)
+rb_big2str0(x, base, trim)
     VALUE x;
     int base;
+    int trim;
 {
     volatile VALUE t;
     BDIGIT *ds;
     long i, j, hbase;
     VALUE ss;
-    char *s, c;
+    char *s;
 
     if (FIXNUM_P(x)) {
 	return rb_fix2str(x, base);
@@ -657,30 +674,30 @@ rb_big2str(x, base)
     switch (base) {
       case 2: break;
       case 3:
-	j = j * 647L / 1024;
+	j = j * 53L / 84 + 1;
 	break;
       case 4: case 5: case 6: case 7:
-	j /= 2;
+	j = (j + 1) / 2;
 	break;
       case 8: case 9:
-	j /= 3;
+	j = (j + 2) / 3;
 	break;
       case 10: case 11: case 12: case 13: case 14: case 15:
-	j = j * 241L / 800;
+	j = j * 28L / 93 + 1;
 	break;
       case 16: case 17: case 18: case 19: case 20: case 21:
       case 22: case 23: case 24: case 25: case 26: case 27:
       case 28: case 29: case 30: case 31:
-	j /= 4;
+	j = (j + 3) / 4;
 	break;
       case 32: case 33: case 34: case 35: case 36:
-	j /= 5;
+	j = (j + 4) / 5;
 	break;
       default:
 	rb_raise(rb_eArgError, "illegal radix %d", base);
 	break;
     }
-    j += 2;
+    j++;			/* space for sign */
 
     hbase = base * base;
 #if SIZEOF_BDIGITS > 2
@@ -689,11 +706,12 @@ rb_big2str(x, base)
 
     t = rb_big_clone(x);
     ds = BDIGITS(t);
-    ss = rb_str_new(0, j);
+    ss = rb_str_new(0, j+1);
     s = RSTRING(ss)->ptr;
 
     s[0] = RBIGNUM(x)->sign ? '+' : '-';
-    while (i && j) {
+    TRAP_BEG;
+    while (i && j > 1) {
 	long k = i;
 	BDIGIT_DBL num = 0;
 
@@ -702,23 +720,37 @@ rb_big2str(x, base)
 	    ds[k] = (BDIGIT)(num / hbase);
 	    num %= hbase;
 	}
-	if (ds[i-1] == 0) i--;
+	if (trim && ds[i-1] == 0) i--;
 	k = SIZEOF_BDIGITS;
 	while (k--) {
-	    c = (char)(num % base);
-	    s[--j] = ruby_digitmap[(int)c];
+	    s[--j] = ruby_digitmap[num % base];
 	    num /= base;
-	    if (i == 0 && num == 0) break;
+	    if (!trim && j <= 1) break;
+	    if (trim && i == 0 && num == 0) break;
 	}
     }
-    while (s[j] == '0') j++;
-    RSTRING(ss)->len -= RBIGNUM(x)->sign?j:j-1;
-    memmove(RBIGNUM(x)->sign?s:s+1, s+j, RSTRING(ss)->len);
+    if (trim) {while (s[j] == '0') j++;}
+    i = RSTRING(ss)->len - j;
+    if (RBIGNUM(x)->sign) {
+	memmove(s, s+j, i);
+	RSTRING(ss)->len = i-1;
+    }
+    else {
+	memmove(s+1, s+j, i);
+	RSTRING(ss)->len = i;
+    }
     s[RSTRING(ss)->len] = '\0';
+    TRAP_END;
 
     return ss;
 }
 
+VALUE
+rb_big2str(VALUE x, int base)
+{
+    return rb_big2str0(x, base, Qtrue);
+}
+
 /*
  *  call-seq:
  *     big.to_s(base=10)   =>  string
@@ -1229,15 +1261,8 @@ rb_big_minus(x, y)
     }
 }
 
-/*
- *  call-seq:
- *     big * other  => Numeric
- *
- *  Multiplies big and other, returning the result.
- */
-
 VALUE
-rb_big_mul(x, y)
+rb_big_mul0(x, y)
     VALUE x, y;
 {
     long i, j;
@@ -1280,7 +1305,21 @@ rb_big_mul(x, y)
 	}
     }
 
-    return bignorm(z);
+    return z;
+}
+
+/*
+ *  call-seq:
+ *     big * other  => Numeric
+ *
+ *  Multiplies big and other, returning the result.
+ */
+
+VALUE
+rb_big_mul(x, y)
+    VALUE x, y;
+{
+    return bignorm(rb_big_mul0(x, y));
 }
 
 static void
@@ -1655,9 +1694,11 @@ rb_big_pow(x, y)
 		if (yy == 0) break;
 		while (yy % 2 == 0) {
 		    yy /= 2;
-		    x = rb_big_mul(x, x);
+		    x = rb_big_mul0(x, x);
+		    bigtrunc(x);
 		}
-		z = rb_big_mul(z, x);
+		z = rb_big_mul0(z, x);
+		bigtrunc(z);
 	    }
 	    return bignorm(z);
 	}
@@ -1986,6 +2027,10 @@ big_rshift(x, shift)
     }
     xds = BDIGITS(x);
     i = RBIGNUM(x)->len; j = i - s1;
+    if (j == 0) {
+	if (RBIGNUM(x)->sign) return INT2FIX(0);
+	else return INT2FIX(-1);
+    }
     z = bignew(j, RBIGNUM(x)->sign);
     if (!RBIGNUM(x)->sign) {
 	num = ((BDIGIT_DBL)~0) << BITSPERDIG;
@@ -2026,29 +2071,39 @@ rb_big_aref(x, y)
     VALUE x, y;
 {
     BDIGIT *xds;
-    int shift;
-    long s1, s2;
+    BDIGIT_DBL num;
+    unsigned long shift;
+    long i, s1, s2;
 
     if (TYPE(y) == T_BIGNUM) {
-	if (!RBIGNUM(y)->sign || RBIGNUM(x)->sign)
+	if (!RBIGNUM(y)->sign)
 	    return INT2FIX(0);
-	return INT2FIX(1);
+	if (RBIGNUM(bigtrunc(y))->len > SIZEOF_LONG/SIZEOF_BDIGITS) {
+	  out_of_range:
+	    return RBIGNUM(x)->sign ? INT2FIX(0) : INT2FIX(1);
+	}
+	shift = big2ulong(y, "long", Qfalse);
+    }
+    else {
+	i = NUM2LONG(y);
+	if (i < 0) return INT2FIX(0);
+	shift = (VALUE)i;
     }
-    shift = NUM2INT(y);
-    if (shift < 0) return INT2FIX(0);
     s1 = shift/BITSPERDIG;
     s2 = shift%BITSPERDIG;
 
+    if (s1 >= RBIGNUM(x)->len) goto out_of_range;
     if (!RBIGNUM(x)->sign) {
-	if (s1 >= RBIGNUM(x)->len) return INT2FIX(1);
-	x = rb_big_clone(x);
-	get2comp(x);
+	xds = BDIGITS(x);
+	i = 0; num = 1;
+	while (num += ~xds[i], ++i <= s1) {
+	    num = BIGDN(num);
+	}
     }
     else {
-	if (s1 >= RBIGNUM(x)->len) return INT2FIX(0);
+	num = BDIGITS(x)[s1];
     }
-    xds = BDIGITS(x);
-    if (xds[s1] & (1<<s2))
+    if (num & ((BDIGIT_DBL)1<<s2))
 	return INT2FIX(1);
     return INT2FIX(0);
 }
