@@ -2,8 +2,8 @@
 
   dln.c -
 
-  $Author: eban $
-  $Date: 2005/11/26 11:01:01 $
+  $Author: shyouhei $
+  $Date: 2007-02-13 08:01:19 +0900 (Tue, 13 Feb 2007) $
   created at: Tue Jan 18 17:05:06 JST 1994
 
   Copyright (C) 1993-2003 Yukihiro Matsumoto
@@ -27,10 +27,6 @@
 char *dln_argv0;
 #endif
 
-#ifdef _AIX
-#pragma alloca
-#endif
-
 #if defined(HAVE_ALLOCA_H)
 #include <alloca.h>
 #endif
@@ -89,8 +85,6 @@ char *getenv();
 # include <image.h>
 #endif
 
-int eaccess();
-
 #ifndef NO_DLN_LOAD
 
 #if defined(HAVE_DLOPEN) && !defined(USE_DLN_A_OUT) && !defined(_AIX) && !defined(__APPLE__) && !defined(_UNICOSMP)
