@@ -2,8 +2,8 @@
 
   enum.c -
 
-  $Author: nobu $
-  $Date: 2005/06/30 16:46:13 $
+  $Author: shyouhei $
+  $Date: 2007-02-12 16:01:19 -0700 (Mon, 12 Feb 2007) $
   created at: Fri Oct  1 15:15:19 JST 1993
 
   Copyright (C) 1993-2003 Yukihiro Matsumoto
@@ -578,7 +578,7 @@ any_i(i, memo)
  *  
  *  Passes each element of the collection to the given block. The method
  *  returns <code>true</code> if the block ever returns a value other
- *  that <code>false</code> or <code>nil</code>. If the block is not
+ *  than <code>false</code> or <code>nil</code>. If the block is not
  *  given, Ruby adds an implicit block of <code>{|obj| obj}</code> (that
  *  is <code>any?</code> will return <code>true</code> if at least one
  *  of the collection members is not <code>false</code> or
