@@ -2,8 +2,8 @@
 
   error.c -
 
-  $Author: matz $
-  $Date: 2006/07/20 07:04:13 $
+  $Author: shyouhei $
+  $Date: 2008-08-03 21:16:55 -0600 (Sun, 03 Aug 2008) $
   created at: Mon Aug  9 16:11:34 JST 1993
 
   Copyright (C) 1993-2003 Yukihiro Matsumoto
@@ -334,7 +334,7 @@ rb_exc_new3(etype, str)
     VALUE etype, str;
 {
     StringValue(str);
-    return rb_exc_new(etype, RSTRING(str)->ptr, RSTRING(str)->len);
+    return rb_funcall(etype, rb_intern("new"), 1, str);
 }
 
 /*
@@ -488,14 +488,14 @@ static VALUE
 exc_backtrace(exc)
     VALUE exc;
 {
-    ID bt = rb_intern("bt");
+    static ID bt;
 
-    if (!rb_ivar_defined(exc, bt)) return Qnil;
-    return rb_ivar_get(exc, bt);
+    if (!bt) bt = rb_intern("bt");
+    return rb_attr_get(exc, bt);
 }
 
-static VALUE
-check_backtrace(bt)
+VALUE
+rb_check_backtrace(bt)
     VALUE bt;
 {
     long i;
@@ -532,7 +532,7 @@ exc_set_backtrace(exc, bt)
     VALUE exc;
     VALUE bt;
 {
-    return rb_iv_set(exc, "bt", check_backtrace(bt));
+    return rb_iv_set(exc, "bt", rb_check_backtrace(bt));
 }
 
 /*
