@@ -1,9 +1,10 @@
 # $RoughId: extconf.rb,v 1.3 2001/08/14 19:54:51 knu Exp $
-# $Id: extconf.rb,v 1.4.2.1 2004/01/21 07:01:43 nobu Exp $
+# $Id: extconf.rb 11708 2007-02-12 23:01:19Z shyouhei $
 
 require "mkmf"
 
-$CFLAGS << " -DHAVE_CONFIG_H -I#{File.dirname(__FILE__)}/.."
+$defs << "-DNDEBUG" << "-DHAVE_CONFIG_H"
+$INCFLAGS << " -I$(srcdir)/.."
 
 $objs = [ "rmd160init.#{$OBJEXT}" ]
 
@@ -13,7 +14,7 @@ if !with_config("bundled-rmd160") &&
     have_library("crypto") && have_header("openssl/ripemd.h")
   $objs << "rmd160ossl.#{$OBJEXT}"
 else
-  $objs << "rmd160.#{$OBJEXT}" << "rmd160hl.#{$OBJEXT}"
+  $objs << "rmd160.#{$OBJEXT}"
 end
 
 have_header("sys/cdefs.h")
@@ -22,4 +23,6 @@ have_header("inttypes.h")
 
 have_header("unistd.h")
 
+$preload = %w[digest]
+
 create_makefile("digest/rmd160")
