@@ -1,13 +1,19 @@
 #! /usr/local/bin/ruby
 # -*- ruby -*-
 
+$extension = nil
+$extstatic = nil
 $force_static = nil
 $install = nil
 $destdir = nil
+$dryrun = false
 $clean = nil
 $nodynamic = nil
 $extinit = nil
 $extobjs = nil
+$extflags = ""
+$extlibs = nil
+$extpath = nil
 $ignore = nil
 $message = nil
 
@@ -79,6 +85,8 @@ def extract_makefile(makefile, keep = true)
     s.sub!(/ *#{Regexp.quote($LIBS)}$/, "")
     $libs = s
   end
+  $objs = (m[/^OBJS[ \t]*=[ \t](.*)/, 1] || "").split
+  $srcs = (m[/^SRCS[ \t]*=[ \t](.*)/, 1] || "").split
   $LOCAL_LIBS = m[/^LOCAL_LIBS[ \t]*=[ \t]*(.*)/, 1] || ""
   $LIBPATH = Shellwords.shellwords(m[/^libpath[ \t]*=[ \t]*(.*)/, 1] || "") - %w[$(libdir) $(topdir)]
   true
@@ -113,6 +121,8 @@ def extmake(target)
     $mdir = target
     $srcdir = File.join($top_srcdir, "ext", $mdir)
     $preload = nil
+    $objs = ""
+    $srcs = ""
     $compiled[target] = false
     makefile = "./Makefile"
     ok = File.exist?(makefile)
@@ -145,18 +155,17 @@ def extmake(target)
       begin
 	$extconf_h = nil
 	ok &&= extract_makefile(makefile)
+	conf = ["#{$srcdir}/makefile.rb", "#{$srcdir}/extconf.rb"].find {|f| File.exist?(f)}
 	if (($extconf_h && !File.exist?($extconf_h)) ||
 	    !(t = modified?(makefile, MTIMES)) ||
-            %W"#{$srcdir}/makefile.rb #{$srcdir}/extconf.rb #{$srcdir}/depend".any? {|f| modified?(f, [t])})
+	    [conf, "#{$srcdir}/depend"].any? {|f| modified?(f, [t])})
         then
 	  ok = false
           init_mkmf
 	  Logging::logfile 'mkmf.log'
 	  rm_f makefile
-	  if File.exist?($0 = "#{$srcdir}/makefile.rb")
-	    load $0
-	  elsif File.exist?($0 = "#{$srcdir}/extconf.rb")
-	    load $0
+	  if conf
+	    load $0 = conf
 	  else
 	    create_makefile(target)
 	  end
@@ -346,7 +355,11 @@ else
   $ruby = '$(topdir)/miniruby' + EXEEXT
 end
 $ruby << " -I'$(topdir)' -I'$(top_srcdir)/lib'"
+$ruby << " -I'$(extout)/$(arch)' -I'$(extout)/common'" if $extout
+$ruby << " -I'$(hdrdir)/ext' -rpurelib.rb"
 $config_h = '$(topdir)/config.h'
+ENV["RUBYLIB"] = "-"
+ENV["RUBYOPT"] = "-rpurelib.rb"
 
 MTIMES = [__FILE__, 'rbconfig.rb', srcdir+'/lib/mkmf.rb'].collect {|f| File.mtime(f)}
 
@@ -395,12 +408,12 @@ else
     elsif (w = w.grep(String)).empty?
       proc {true}
     else
-      w.collect {|opt| opt.split(/,/)}.flatten.method(:any?)
+      proc {|c1| w.collect {|opt| opt.split(/,/)}.flatten.any?(&c1)}
     end
   }
   cond = proc {|ext|
     cond1 = proc {|n| File.fnmatch(n, ext, File::FNM_PATHNAME)}
-    withes.call(&cond1) or !withouts.call(&cond1)
+    withes.call(cond1) or !withouts.call(cond1)
   }
   exts |= Dir.glob("#{ext_prefix}/*/**/extconf.rb").collect {|d|
     d = File.dirname(d)
@@ -412,15 +425,7 @@ else
 end
 
 if $extout
-  Config.expand(extout = "#$extout", Config::CONFIG.merge("topdir"=>$topdir))
-  if $install
-    dest = Config.expand($rubylibdir.dup)
-    unless $destdir.empty?
-      dest.sub!($dest_prefix_pattern, Config.expand($destdir.dup))
-    end
-    FileUtils.cp_r(extout+"/.", dest, :verbose => true, :noop => $dryrun)
-    exit
-  end
+  extout = Config.expand("#{$extout}", Config::CONFIG.merge("topdir"=>$topdir))
   unless $ignore
     FileUtils.mkpath(extout)
   end
@@ -454,9 +459,13 @@ if $ignore
   exit
 end
 
-if $extlist.size > 0
-  $extinit ||= ""
-  $extobjs ||= ""
+$extinit ||= ""
+$extobjs ||= ""
+$extpath ||= []
+$extflags ||= ""
+$extlibs ||= []
+unless $extlist.empty?
+  $extinit << "\n" unless $extinit.empty?
   list = $extlist.dup
   built = []
   while e = list.shift
@@ -470,28 +479,33 @@ if $extlist.size > 0
     end
     f = format("%s/%s.%s", s, i, $LIBEXT)
     if File.exist?(f)
-      $extinit += "\tinit(Init_#{i}, \"#{t}.so\");\n"
-      $extobjs += "ext/#{f} "
+      $extinit << "    init(Init_#{i}, \"#{t}.so\");\n"
+      $extobjs << "ext/#{f} "
       built << t
     end
   end
 
   src = %{\
-extern char *ruby_sourcefile, *rb_source_filename();
-#define init(func, name) (ruby_sourcefile = src = rb_source_filename(name), func(), rb_provide(src))
-void Init_ext() {\n\tchar* src;\n#$extinit}
+#include "ruby.h"
+
+#define init(func, name) {void func _((void)); ruby_init_ext(name, func);}
+
+void ruby_init_ext _((const char *name, void (*init)(void)));
+
+void Init_ext _((void))\n{\n#$extinit}
 }
   if !modified?(extinit.c, MTIMES) || IO.read(extinit.c) != src
     open(extinit.c, "w") {|f| f.print src}
   end
 
-  $extobjs = "ext/#{extinit.o} " + $extobjs
+  $extobjs = "ext/#{extinit.o} #{$extobjs}"
   if RUBY_PLATFORM =~ /m68k-human|beos/
     $extflags.delete("-L/usr/local/lib")
   end
   $extpath.delete("$(topdir)")
   $extflags = libpathflag($extpath) << " " << $extflags.strip
   conf = [
+    ['LIBRUBY_SO_UPDATE', '$(LIBRUBY_EXTS)'],
     ['SETUP', $setup],
     [enable_config("shared", $enable_shared) ? 'DLDOBJS' : 'EXTOBJS', $extobjs],
     ['EXTLIBS', $extlibs.join(' ')], ['EXTLDFLAGS', $extflags]
@@ -508,7 +522,7 @@ rubies = []
 %w[RUBY RUBYW STATIC_RUBY].each {|r|
   n = r
   if r = arg_config("--"+r.downcase) || config_string(r+"_INSTALL_NAME")
-    rubies << r+EXEEXT
+    rubies << Config.expand(r+=EXEEXT)
     $mflags << "#{n}=#{r}"
   end
 }
@@ -517,15 +531,21 @@ Dir.chdir ".."
 unless $destdir.to_s.empty?
   $mflags.defined?("DESTDIR") or $mflags << "DESTDIR=#{$destdir}"
 end
-unless $extlist.empty?
-  rm_f(Config::CONFIG["LIBRUBY_SO"])
-end
 puts "making #{rubies.join(', ')}"
 $stdout.flush
 $mflags.concat(rubies)
 
 if $nmake == ?b
-  $mflags.collect {|flag| flag.sub!(/\A(?=\w+=)/, "-D")}
+  unless (vars = $mflags.grep(/\A\w+=/n)).empty?
+    open(mkf = "libruby.mk", "wb") do |tmf|
+      tmf.puts("!include Makefile")
+      tmf.puts
+      tmf.puts(*vars.map {|v| v.sub(/=/, " = ")})
+      tmf.puts("PRE_LIBRUBY_UPDATE = del #{mkf}")
+    end
+    $mflags.unshift("-f#{mkf}")
+    vars.each {|flag| flag.sub!(/\A/, "-D")}
+  end
 end
 system($make, *sysquote($mflags)) or exit($?.exitstatus)
 
