@@ -3,8 +3,8 @@
 
   iconv.c -
 
-  $Author: matz $
-  $Date: 2005/12/12 00:36:51 $
+  $Author: shyouhei $
+  $Date: 2008-06-15 19:20:39 +0900 (Sun, 15 Jun 2008) $
   created at: Wed Dec  1 20:28:09 JST 1999
 
   All the files in this distribution are covered under the Ruby's
@@ -652,7 +652,7 @@ iconv_s_iconv
 
 /*
  * Document-method: Iconv::conv
- * call-seq: Iconv.iconv(to, from, *strs)
+ * call-seq: Iconv.conv(to, from, str)
  *
  * Shorthand for
  *   Iconv.iconv(to, from, str).join
