@@ -1,5 +1,5 @@
 /*
- * $Id: ossl_asn1.c,v 1.5.2.8 2005/09/10 01:11:15 gotoyuzo Exp $
+ * $Id: ossl_asn1.c 12043 2007-03-12 04:12:32Z knu $
  * 'OpenSSL for Ruby' team members
  * Copyright (C) 2003
  * All rights reserved.
@@ -766,7 +766,7 @@ ossl_asn1_decode0(unsigned char **pp, long length, long *offset, long depth,
 	if(tag_class == sUNIVERSAL &&
 	   tag < ossl_asn1_info_size && ossl_asn1_info[tag].klass){
 	    VALUE klass = *ossl_asn1_info[tag].klass;
-	    long flag;
+	    long flag = 0;
 	    if(!rb_obj_is_kind_of(value, rb_cArray)){
 		switch(tag){
 		case V_ASN1_BOOLEAN:
@@ -1087,6 +1087,10 @@ Init_ossl_asn1()
     VALUE ary;
     int i;
 
+#if 0 /* let rdoc know about mOSSL */
+    mOSSL = rb_define_module("OpenSSL");
+#endif
+
     sUNIVERSAL = rb_intern("UNIVERSAL");
     sCONTEXT_SPECIFIC = rb_intern("CONTEXT_SPECIFIC");
     sAPPLICATION = rb_intern("APPLICATION");
