@@ -8,7 +8,7 @@
  */
 
 #ifndef lint
-/*char sdbm_rcsid[] = "$Id: _sdbm.c,v 1.5.2.1 2005/09/08 05:59:41 matz Exp $";*/
+/*char sdbm_rcsid[] = "$Id: _sdbm.c 11708 2007-02-12 23:01:19Z shyouhei $";*/
 #endif
 
 #include "sdbm.h"
@@ -71,7 +71,7 @@ static int   duppair proto((char *, datum));
 
 #include <stdio.h>
 #include <stdlib.h>
-#ifdef DOSISH
+#ifdef MSDOS
 #include <io.h>
 #endif
 #include <sys/types.h>
@@ -626,7 +626,7 @@ register DBM *db;
  */
 
 #ifndef lint
-/*char pair_rcsid[] = "$Id: _sdbm.c,v 1.5.2.1 2005/09/08 05:59:41 matz Exp $";*/
+/*char pair_rcsid[] = "$Id: _sdbm.c 11708 2007-02-12 23:01:19Z shyouhei $";*/
 #endif
 
 #ifndef BSD42
