@@ -6,7 +6,11 @@ when /bccwin32/
   have_library("ws2_32", "WSACleanup")
 when /mswin32|mingw/
   test_func = "WSACleanup"
-  have_library("wsock32", "WSACleanup")
+  if with_config("winsock2")
+    have_library("ws2_32", "WSACleanup")
+  else
+    have_library("wsock32", "WSACleanup")
+  end
 when /cygwin/
   test_func = "socket"
 when /beos/
@@ -29,8 +33,11 @@ if /solaris/ =~ RUBY_PLATFORM and !try_compile("")
   # bug of gcc 3.0 on Solaris 8 ?
   headers << "sys/feature_tests.h"
 end
+if have_header("arpa/inet.h")
+  headers << "arpa/inet.h"
+end
 
-$ipv6 = false
+ipv6 = false
 default_ipv6 = /cygwin/ !~ RUBY_PLATFORM
 if enable_config("ipv6", default_ipv6)
   if checking_for("ipv6") {try_link(<<EOF)}
@@ -41,74 +48,45 @@ main()
   socket(AF_INET6, SOCK_STREAM, 0);
 }
 EOF
-    $CFLAGS+=" -DENABLE_IPV6"
-    $ipv6 = true
+    $defs << "-DENABLE_IPV6" << "-DINET6"
+    ipv6 = true
   end
 end
 
-$ipv6type = nil
-$ipv6lib = nil
-$ipv6libdir = nil
-$ipv6trylibc = nil
-if $ipv6
-  if macro_defined?("IPV6_INRIA_VERSION", <<EOF)
-#include <netinet/in.h>
-EOF
-    $ipv6type = "inria"
-    $CFLAGS="-DINET6 "+$CFLAGS
-  elsif macro_defined?("__KAME__", <<EOF)
-#include <netinet/in.h>
-EOF
-    $ipv6type = "kame"
-    $ipv6lib="inet6"
-    $ipv6libdir="/usr/local/v6/lib"
-    $ipv6trylibc=true
-    $CFLAGS="-DINET6 "+$CFLAGS
-  elsif File.directory? "/usr/inet6"
-    $ipv6type = "linux"
-    $ipv6lib="inet6"
-    $ipv6libdir="/usr/inet6/lib"
-    $CFLAGS="-DINET6 -I/usr/inet6/include "+$CFLAGS
-  elsif macro_defined?("_TOSHIBA_INET6", <<EOF)
-#include <sys/param.h>
-EOF
-    $ipv6type = "toshiba"
-    $ipv6lib="inet6"
-    $ipv6libdir="/usr/local/v6/lib"
-    $CFLAGS="-DINET6 "+$CFLAGS
-  elsif macro_defined?("__V6D__", <<EOF)
-#include </usr/local/v6/include/sys/v6config.h>
-EOF
-    $ipv6type = "v6d"
-    $ipv6lib="v6"
-    $ipv6libdir="/usr/local/v6/lib"
-    $CFLAGS="-DINET6 -I/usr/local/v6/include "+$CFLAGS
-  elsif macro_defined?("_ZETA_MINAMI_INET6", <<EOF)
-#include <sys/param.h>
-EOF
-    $ipv6type = "zeta"
-    $ipv6lib="inet6"
-    $ipv6libdir="/usr/local/v6/lib"
-    $CFLAGS="-DINET6 "+$CFLAGS
-  else
-    $ipv6lib=with_config("ipv6-lib", nil)
-    $ipv6libdir=with_config("ipv6-libdir", nil)
-    $CFLAGS="-DINET6 "+$CFLAGS
+if ipv6
+  ipv6lib = nil
+  class << (fmt = "unknown")
+    def %(s) s || self end
   end
-  
-  if $ipv6lib
-    if File.directory? $ipv6libdir and File.exist? "#{$ipv6libdir}/lib#{$ipv6lib}.a"
-      $LOCAL_LIBS = " -L#$ipv6libdir -l#$ipv6lib"
-    elsif !$ipv6trylibc
-      print <<EOS
+  idirs, ldirs = dir_config("inet6", %w[/usr/inet6 /usr/local/v6].find {|d| File.directory?(d)})
+  checking_for("ipv6 type", fmt) do
+    if have_macro("IPV6_INRIA_VERSION", "netinet/in.h")
+      "inria"
+    elsif have_macro("__KAME__", "netinet/in.h")
+      have_library(ipv6lib = "inet6")
+      "kame"
+    elsif have_macro("_TOSHIBA_INET6", "sys/param.h")
+      have_library(ipv6lib = "inet6") and "toshiba"
+    elsif have_macro("__V6D__", "sys/v6config.h")
+      have_library(ipv6lib = "v6") and "v6d"
+    elsif have_macro("_ZETA_MINAMI_INET6", "sys/param.h")
+      have_library(ipv6lib = "inet6") and "zeta"
+    elsif ipv6lib = with_config("ipv6-lib")
+      warn <<EOS
+--with-ipv6-lib and --with-ipv6-libdir option will be obsolete, use
+--with-inet6lib and --with-inet6-{include,lib} options instead.
+EOS
+      find_library(ipv6lib, nil, with_config("ipv6-libdir", ldirs)) and
+        ipv6lib
+    elsif have_library("inet6")
+      "inet6"
+    end
+  end or not ipv6lib or abort <<EOS
 
-Fatal: no #$ipv6lib library found.  cannot continue.
-You need to fetch lib#{$ipv6lib}.a from appropriate
+Fatal: no #{ipv6lib} library found.  cannot continue.
+You need to fetch lib#{ipv6lib}.a from appropriate
 ipv6 kit and compile beforehand.
 EOS
-      exit
-    end
-  end
 end
 
 if have_struct_member("struct sockaddr_in", "sin_len", headers)
@@ -178,6 +156,10 @@ main()
       if (ai->ai_addr == NULL)
         goto bad;
 #if defined(_AIX)
+      if (ai->ai_family == AF_INET6 && passive) {
+        inet6++;
+        continue;
+      }
       ai->ai_addr->sa_len = ai->ai_addrlen;
       ai->ai_addr->sa_family = ai->ai_family;
 #endif
@@ -241,7 +223,7 @@ main()
 }
 EOF
 end
-if $ipv6 and not getaddr_info_ok
+if ipv6 and not getaddr_info_ok
   abort <<EOS
 
 Fatal: --enable-ipv6 is specified, and your OS seems to support IPv6 feature.
@@ -277,7 +259,19 @@ unless getaddr_info_ok and have_func("getnameinfo", "netdb.h") and have_func("ge
   have_func("inet_ntop") or have_func("inet_ntoa")
   have_func("inet_pton") or have_func("inet_aton")
   have_func("getservbyport")
-  have_header("arpa/inet.h")
+  if have_func("gai_strerror")
+    unless checking_for("gai_strerror() returns const pointer") {!try_compile(<<EOF)}
+#{cpp_include(headers)}
+#include <stdlib.h>
+void
+conftest_gai_strerror_is_const()
+{
+    *gai_strerror(0) = 0;
+}
+EOF
+      $defs << "-DGAI_STRERROR_CONST"
+    end
+  end
   have_header("arpa/nameser.h")
   have_header("resolv.h")
 end
