@@ -2,7 +2,7 @@
 #
 # benchmark.rb - a performance benchmarking library 
 # 
-# $Id: benchmark.rb,v 1.8.2.2 2004/04/18 23:20:32 nobu Exp $
+# $Id: benchmark.rb 11708 2007-02-12 23:01:19Z shyouhei $
 # 
 # Created by Gotoken (gotoken@notwork.org). 
 #
@@ -304,10 +304,7 @@ module Benchmark
   # Returns the elapsed real time used to execute the given block.
   #
   def realtime(&blk) # :yield:
-    r0 = Time.now
-    yield
-    r1 = Time.now
-    r1.to_f - r0.to_f
+    Benchmark::measure(&blk).real
   end
 
 
@@ -333,7 +330,7 @@ module Benchmark
     # Registers the given label and block pair in the job list.
     #
     def item(label = "", &blk) # :yield:
-      raise ArgumentError, "no block" unless block_given?
+      raise ArgmentError, "no block" unless block_given?
       label.concat ' '
       w = label.length
       @width = w if @width < w
