@@ -14,6 +14,7 @@ if /mswin|bccwin|mingw|msdosdjgpp|human|os2/ !~ CONFIG['build_os']
 end
 SRC_EXT = %w[c m] << CXX_EXT
 $static = $config_h = nil
+$default_static = $static
 
 unless defined? $configure_args
   $configure_args = {}
@@ -60,13 +61,14 @@ $netbsd = /netbsd/ =~ RUBY_PLATFORM
 $os2 = /os2/ =~ RUBY_PLATFORM
 $beos = /beos/ =~ RUBY_PLATFORM
 $solaris = /solaris/ =~ RUBY_PLATFORM
+$dest_prefix_pattern = (File::PATH_SEPARATOR == ';' ? /\A([[:alpha:]]:)?/ : /\A/)
 
 def config_string(key, config = CONFIG)
   s = config[key] and !s.empty? and block_given? ? yield(s) : s
 end
 
 def dir_re(dir)
-  Regexp.new('\$(?:\('+dir+'\)|\{'+dir+'\})(?:\$\(target_prefix\)|\{target_prefix\})?')
+  Regexp.new('\$(?:\('+dir+'\)|\{'+dir+'\})(?:\$(?:\(target_prefix\)|\{target_prefix\}))?')
 end
 
 INSTALL_DIRS = [
@@ -81,8 +83,8 @@ INSTALL_DIRS = [
 def install_dirs(target_prefix = nil)
   if $extout
     dirs = [
-      ['RUBYCOMMONDIR', '$(extout)'],
-      ['RUBYLIBDIR',    '$(extout)$(target_prefix)'],
+      ['RUBYCOMMONDIR', '$(extout)/common'],
+      ['RUBYLIBDIR',    '$(RUBYCOMMONDIR)$(target_prefix)'],
       ['RUBYARCHDIR',   '$(extout)/$(arch)$(target_prefix)'],
       ['extout',        "#$extout"],
       ['extout_prefix', "#$extout_prefix"],
@@ -234,17 +236,22 @@ def xpopen command, *mode, &block
 end
 
 def log_src(src)
-  Logging::message <<"EOM", src
+  src = src.split(/^/)
+  fmt = "%#{src.size.to_s.size}d: %s"
+  Logging::message <<"EOM"
 checked program was:
 /* begin */
-%s/* end */
+EOM
+  src.each_with_index {|line, no| Logging::message fmt, no+1, line}
+  Logging::message <<"EOM"
+/* end */
 
 EOM
 end
 
 def create_tmpsrc(src)
   src = yield(src) if block_given?
-  src = src.sub(/[^\n]\z/, "\\&\n")
+  src = src.gsub(/[ \t]+$/, '').gsub(/\A\n+|^\n+$/, '').sub(/[^\n]\z/, "\\&\n")
   open(CONFTEST_C, "wb") do |cfile|
     cfile.print src
   end
@@ -258,7 +265,7 @@ ensure
   log_src(src)
 end
 
-def link_command(ldflags, opt="", libpath=$LIBPATH)
+def link_command(ldflags, opt="", libpath=$DEFLIBPATH|$LIBPATH)
   Config::expand(TRY_LINK.dup,
                  CONFIG.merge('hdrdir' => $hdrdir.quote,
                               'src' => CONFTEST_C,
@@ -273,18 +280,16 @@ def link_command(ldflags, opt="", libpath=$LIBPATH)
 end
 
 def cc_command(opt="")
-  Config::expand("$(CC) -c #$INCFLAGS -I$(hdrdir) " \
-                 "#$CPPFLAGS #$CFLAGS #$ARCH_FLAG #{opt} #{CONFTEST_C}",
-		 CONFIG.merge('hdrdir' => $hdrdir.quote))
+  Config::expand("$(CC) #$INCFLAGS #$CPPFLAGS #$CFLAGS #$ARCH_FLAG #{opt} -c #{CONFTEST_C}",
+		 CONFIG.merge('hdrdir' => $hdrdir.quote, 'srcdir' => $srcdir.quote))
 end
 
 def cpp_command(outfile, opt="")
-  Config::expand("$(CPP) #$INCFLAGS -I$(hdrdir) " \
-                 "#$CPPFLAGS #$CFLAGS #{opt} #{CONFTEST_C} #{outfile}",
-		 CONFIG.merge('hdrdir' => $hdrdir.quote))
+  Config::expand("$(CPP) #$INCFLAGS #$CPPFLAGS #$CFLAGS #{opt} #{CONFTEST_C} #{outfile}",
+		 CONFIG.merge('hdrdir' => $hdrdir.quote, 'srcdir' => $srcdir.quote))
 end
 
-def libpathflag(libpath=$LIBPATH)
+def libpathflag(libpath=$DEFLIBPATH|$LIBPATH)
   libpath.map{|x|
     (x == "$(topdir)" ? LIBPATHFLAG : LIBPATHFLAG+RPATHFLAG) % x.quote
   }.join
@@ -370,6 +375,7 @@ def try_constant(const, headers = nil, opt = "", &b)
       return nil
     end
     upper = 1
+    lower = 0
     until try_static_assert("#{const} <= #{upper}", headers, opt)
       lower = upper
       upper <<= 1
@@ -383,11 +389,6 @@ def try_constant(const, headers = nil, opt = "", &b)
         upper = mid
       end
     end
-    unless upper == lower
-      if try_static_assert("#{const} == #{lower}", headers, opt)
-        upper = lower
-      end
-    end
     upper = -upper if neg
     return upper
   else
@@ -410,17 +411,17 @@ end
 def try_func(func, libs, headers = nil, &b)
   headers = cpp_include(headers)
   try_link(<<"SRC", libs, &b) or try_link(<<"SRC", libs, &b)
-#{headers}
-/*top*/
-int main() { return 0; }
-int t() { #{func}(); return 0; }
-SRC
 #{COMMON_HEADERS}
 #{headers}
 /*top*/
 int main() { return 0; }
 int t() { void ((*volatile p)()); p = (void ((*)()))#{func}; return 0; }
 SRC
+#{headers}
+/*top*/
+int main() { return 0; }
+int t() { #{func}(); return 0; }
+SRC
 end
 
 def try_var(var, headers = nil, &b)
@@ -430,7 +431,7 @@ def try_var(var, headers = nil, &b)
 #{headers}
 /*top*/
 int main() { return 0; }
-int t() { void *volatile p; p = (void *)&#{var}; return 0; }
+int t() { const volatile void *volatile p; p = (void *)&#{var}; return 0; }
 SRC
 end
 
@@ -535,7 +536,7 @@ end
 
 def checking_for(m, fmt = nil)
   f = caller[0][/in `(.*)'$/, 1] and f << ": " #` for vim
-  m = "checking for #{m}... "
+  m = "checking #{/\Acheck/ =~ f ? '' : 'for '}#{m}... "
   message "%s", m
   a = r = nil
   Logging::postpone do
@@ -548,23 +549,51 @@ def checking_for(m, fmt = nil)
   r
 end
 
+def checking_message(target, place = nil, opt = nil)
+  [["in", place], ["with", opt]].inject("#{target}") do |msg, (pre, noun)|
+    if noun
+      [[:to_str], [:join, ","], [:to_s]].each do |meth, *args|
+        if noun.respond_to?(meth)
+          break noun = noun.send(meth, *args)
+        end
+      end
+      msg << " #{pre} #{noun}" unless noun.empty?
+    end
+    msg
+  end
+end
+
+# Returns whether or not +macro+ is defined either in the common header
+# files or within any +headers+ you provide.
+#
+# Any options you pass to +opt+ are passed along to the compiler.
+#
 def have_macro(macro, headers = nil, opt = "", &b)
-  m = "#{macro}"
-  m << " in #{headers.inspect}" if headers
-  checking_for m do
+  checking_for checking_message(macro, headers, opt) do
     macro_defined?(macro, cpp_include(headers), opt, &b)
   end
 end
 
-def have_library(lib, func = nil, header=nil, &b)
+# Returns whether or not the given entry point +func+ can be found within
+# +lib+.  If +func+ is nil, the 'main()' entry point is used by default.
+# If found, it adds the library to list of libraries to be used when linking
+# your extension.
+#
+# If +headers+ are provided, it will include those header files as the
+# header files it looks in when searching for +func+.
+#
+# Real name of the library to be linked can be altered by
+# '--with-FOOlib' configuration option.
+#
+def have_library(lib, func = nil, headers = nil, &b)
   func = "main" if !func or func.empty?
   lib = with_config(lib+'lib', lib)
-  checking_for "#{func}() in #{LIBARG%lib}" do
+  checking_for checking_message("#{func}()", LIBARG%lib) do
     if COMMON_LIBS.include?(lib)
       true
     else
       libs = append_library($libs, lib)
-      if try_func(func, libs, header, &b)
+      if try_func(func, libs, headers, &b)
         $libs = libs
         true
       else
@@ -574,6 +603,13 @@ def have_library(lib, func = nil, header=nil, &b)
   end
 end
 
+# Returns whether or not the entry point +func+ can be found within the library
+# +lib+ in one of the +paths+ specified, where +paths+ is an array of strings.
+# If +func+ is nil , then the main() function is used as the entry point.
+#
+# If +lib+ is found, then the path it was found on is added to the list of
+# library paths searched and linked against.
+#
 def find_library(lib, func, *paths, &b)
   func = "main" if !func or func.empty?
   lib = with_config(lib+'lib', lib)
@@ -596,8 +632,16 @@ def find_library(lib, func, *paths, &b)
   end
 end
 
+# Returns whether or not the function +func+ can be found in the common
+# header files, or within any +headers+ that you provide.  If found, a
+# macro is passed as a preprocessor constant to the compiler using the
+# function name, in uppercase, prepended with 'HAVE_'.
+#
+# For example, if have_func('foo') returned true, then the HAVE_FOO
+# preprocessor macro would be passed to the compiler.
+#
 def have_func(func, headers = nil, &b)
-  checking_for "#{func}()" do
+  checking_for checking_message("#{func}()", headers) do
     if try_func(func, $libs, headers, &b)
       $defs.push(format("-DHAVE_%s", func.upcase))
       true
@@ -607,8 +651,16 @@ def have_func(func, headers = nil, &b)
   end
 end
 
+# Returns whether or not the variable +var+ can be found in the common
+# header files, or within any +headers+ that you provide.  If found, a
+# macro is passed as a preprocessor constant to the compiler using the
+# variable name, in uppercase, prepended with 'HAVE_'.
+#
+# For example, if have_var('foo') returned true, then the HAVE_FOO
+# preprocessor macro would be passed to the compiler.
+#
 def have_var(var, headers = nil, &b)
-  checking_for "#{var}" do
+  checking_for checking_message(var, headers) do
     if try_var(var, headers, &b)
       $defs.push(format("-DHAVE_%s", var.upcase))
       true
@@ -618,6 +670,13 @@ def have_var(var, headers = nil, &b)
   end
 end
 
+# Returns whether or not the given +header+ file can be found on your system.
+# If found, a macro is passed as a preprocessor constant to the compiler using
+# the header file name, in uppercase, prepended with 'HAVE_'.
+#
+# For example, if have_header('foo.h') returned true, then the HAVE_FOO_H
+# preprocessor macro would be passed to the compiler.
+#
 def have_header(header, &b)
   checking_for header do
     if try_cpp(cpp_include(header), &b)
@@ -629,15 +688,22 @@ def have_header(header, &b)
   end
 end
 
+# Instructs mkmf to search for the given +header+ in any of the +paths+
+# provided, and returns whether or not it was found in those paths.
+#
+# If the header is found then the path it was found on is added to the list
+# of included directories that are sent to the compiler (via the -I switch).
+#
 def find_header(header, *paths)
+  header = cpp_include(header)
   checking_for header do
-    if try_cpp(cpp_include(header))
+    if try_cpp(header)
       true
     else
       found = false
       paths.each do |dir|
         opt = "-I#{dir}".quote
-        if try_cpp(cpp_include(header), opt)
+        if try_cpp(header, opt)
           $INCFLAGS << " " << opt
           found = true
           break
@@ -648,11 +714,22 @@ def find_header(header, *paths)
   end
 end
 
-def have_struct_member(type, member, header = nil, &b)
-  checking_for "#{type}.#{member}" do
+# Returns whether or not the struct of type +type+ contains +member+.  If
+# it does not, or the struct type can't be found, then false is returned.  You
+# may optionally specify additional +headers+ in which to look for the struct
+# (in addition to the common header files).
+#
+# If found, a macro is passed as a preprocessor constant to the compiler using
+# the member name, in uppercase, prepended with 'HAVE_ST_'.
+#
+# For example, if have_struct_member('foo', 'bar') returned true, then the
+# HAVE_ST_BAR preprocessor macro would be passed to the compiler.
+# 
+def have_struct_member(type, member, headers = nil, &b)
+  checking_for checking_message("#{type}.#{member}", headers) do
     if try_compile(<<"SRC", &b)
 #{COMMON_HEADERS}
-#{cpp_include(header)}
+#{cpp_include(headers)}
 /*top*/
 int main() { return 0; }
 int s = (char *)&((#{type}*)0)->#{member} - (char *)0;
@@ -665,19 +742,28 @@ SRC
   end
 end
 
-def have_type(type, header = nil, opt = "", &b)
-  checking_for type do
-    header = cpp_include(header)
-    if try_compile(<<"SRC", opt, &b) or (/\A\w+\z/n =~ type && try_compile(<<"SRC", opt, &b))
+# Returns whether or not the static type +type+ is defined.  You may
+# optionally pass additional +headers+ to check against in addition to the
+# common header files.
+#
+# You may also pass additional flags to +opt+ which are then passed along to
+# the compiler.
+#
+# If found, a macro is passed as a preprocessor constant to the compiler using
+# the type name, in uppercase, prepended with 'HAVE_TYPE_'.
+#
+# For example, if have_type('foo') returned true, then the HAVE_TYPE_FOO
+# preprocessor macro would be passed to the compiler.
+#
+def have_type(type, headers = nil, opt = "", &b)
+  checking_for checking_message(type, headers, opt) do
+    headers = cpp_include(headers)
+    if try_compile(<<"SRC", opt, &b)
 #{COMMON_HEADERS}
-#{header}
+#{headers}
 /*top*/
-static #{type} t;
-SRC
-#{COMMON_HEADERS}
-#{header}
-/*top*/
-static #{type} *t;
+typedef #{type} conftest_type;
+static conftest_type conftestval[sizeof(conftest_type)?1:-1];
 SRC
       $defs.push(format("-DHAVE_TYPE_%s", type.strip.upcase.tr_s("^A-Z0-9_", "_")))
       true
@@ -687,23 +773,28 @@ SRC
   end
 end
 
-def check_sizeof(type, header = nil, &b)
+# Returns the size of the given +type+.  You may optionally specify additional
+# +headers+ to search in for the +type+.
+#
+# If found, a macro is passed as a preprocessor constant to the compiler using
+# the type name, in uppercase, prepended with 'SIZEOF_', followed by the type
+# name, followed by '=X' where 'X' is the actual size.
+#
+# For example, if check_sizeof('mystruct') returned 12, then the
+# SIZEOF_MYSTRUCT=12 preprocessor macro would be passed to the compiler.
+#
+def check_sizeof(type, headers = nil, &b)
   expr = "sizeof(#{type})"
-  m = "checking size of #{type}... "
-  message "%s", m
-  a = size = nil
-  Logging::postpone do
-    if size = try_constant(expr, header, &b)
+  fmt = "%d"
+  def fmt.%(x)
+    x ? super : "failed"
+  end
+  checking_for checking_message("size of #{type}", headers), fmt do
+    if size = try_constant(expr, headers, &b)
       $defs.push(format("-DSIZEOF_%s=%d", type.upcase.tr_s("^A-Z0-9_", "_"), size))
-      a = "#{size}\n"
-    else
-      a = "failed\n"
+      size
     end
-    "check_sizeof: #{m}-------------------- #{a}\n"
   end
-  message(a)
-  Logging::message "--------------------\n\n"
-  size
 end
 
 def scalar_ptr_type?(type, member = nil, headers = nil, &b)
@@ -735,12 +826,11 @@ def what_type?(type, member = nil, headers = nil, &b)
     m << "." << member
     name = "(((#{type} *)0)->#{member})"
   end
-  m << " in #{headers.inspect}" if headers
   fmt = "seems %s"
   def fmt.%(x)
     x ? super : "unknown"
   end
-  checking_for m, fmt do
+  checking_for checking_message(m, headers), fmt do
     if scalar_ptr_type?(type, member, headers, &b)
       if try_static_assert("sizeof(*#{name}) == 1", headers)
         "string"
@@ -765,7 +855,7 @@ def find_executable0(bin, path = nil)
   ext = config_string('EXEEXT')
   if File.expand_path(bin) == bin
     return bin if File.executable?(bin)
-    return file if ext and File.executable?(file = bin + ext)
+    ext and File.executable?(file = bin + ext) and return file
     return nil
   end
   if path ||= ENV['PATH']
@@ -782,7 +872,7 @@ def find_executable0(bin, path = nil)
 end
 
 def find_executable(bin, path = nil)
-  checking_for bin do
+  checking_for checking_message(bin, path) do
     find_executable0(bin, path)
   end
 end
@@ -828,23 +918,37 @@ end
 
 def create_header(header = "extconf.h")
   message "creating %s\n", header
-  if $defs.length > 0
-    sym = header.tr("a-z./\055", "A-Z___")
-    open(header, "w") do |hfile|
-      hfile.print "#ifndef #{sym}\n#define #{sym}\n"
-      for line in $defs
-	case line
-	when /^-D([^=]+)(?:=(.*))?/
-	  hfile.print "#define #$1 #{$2 || 1}\n"
-	when /^-U(.*)/
-	  hfile.print "#undef #$1\n"
-	end
-      end
-      hfile.print "#endif\n"
+  sym = header.tr("a-z./\055", "A-Z___")
+  hdr = ["#ifndef #{sym}\n#define #{sym}\n"]
+  for line in $defs
+    case line
+    when /^-D([^=]+)(?:=(.*))?/
+      hdr << "#define #$1 #{$2 ? Shellwords.shellwords($2)[0] : 1}\n"
+    when /^-U(.*)/
+      hdr << "#undef #$1\n"
     end
   end
+  hdr << "#endif\n"
+  hdr = hdr.join
+  unless (IO.read(header) == hdr rescue false)
+    open(header, "w") do |hfile|
+      hfile.write(hdr)
+    end
+  end
+  $extconf_h = header
 end
 
+# Sets a +target+ name that the user can then use to configure various 'with'
+# options with on the command line by using that name.  For example, if the
+# target is set to "foo", then the user could use the --with-foo-dir command
+# line option.
+#
+# You may pass along additional 'include' or 'lib' defaults via the +idefault+
+# and +ldefault+ parameters, respectively.
+#
+# Note that dir_config only adds to the list of places to search for libraries
+# and include files.  It does not link the libraries into your application.
+#
 def dir_config(target, idefault=nil, ldefault=nil)
   if dir = with_config(target + "-dir", (idefault unless ldefault))
     defaults = Array === dir ? dir : dir.split(File::PATH_SEPARATOR)
@@ -880,16 +984,23 @@ def dir_config(target, idefault=nil, ldefault=nil)
 end
 
 def pkg_config(pkg)
-  unless defined?($PKGCONFIG)
-    if pkgconfig = with_config("pkg-config", !CROSS_COMPILING && "pkg-config")
-      find_executable0(pkgconfig) or pkgconfig = nil
-    end
-    $PKGCONFIG = pkgconfig
+  if pkgconfig = with_config("#{pkg}-config") and find_executable0(pkgconfig)
+    # iff package specific config command is given
+    get = proc {|opt| `#{pkgconfig} --#{opt}`.chomp}
+  elsif ($PKGCONFIG ||= 
+         (pkgconfig = with_config("pkg-config", ("pkg-config" unless CROSS_COMPILING))) &&
+         find_executable0(pkgconfig) && pkgconfig) and
+      system("#{$PKGCONFIG} --exists #{pkg}")
+    # default to pkg-config command
+    get = proc {|opt| `#{$PKGCONFIG} --#{opt} #{pkg}`.chomp}
+  elsif find_executable0(pkgconfig = "#{pkg}-config")
+    # default to package specific config command, as a last resort.
+    get = proc {|opt| `#{pkgconfig} --#{opt}`.chomp}
   end
-  if $PKGCONFIG and system("#{$PKGCONFIG} --exists #{pkg}")
-    cflags = `#{$PKGCONFIG} --cflags #{pkg}`.chomp
-    ldflags = `#{$PKGCONFIG} --libs #{pkg}`.chomp
-    libs = `#{$PKGCONFIG} --libs-only-l #{pkg}`.chomp
+  if get
+    cflags = get['cflags']
+    ldflags = get['libs']
+    libs = get['libs-only-l']
     ldflags = (Shellwords.shellwords(ldflags) - Shellwords.shellwords(libs)).quote.join(" ")
     $CFLAGS += " " << cflags
     $LDFLAGS += " " << ldflags
@@ -898,11 +1009,15 @@ def pkg_config(pkg)
     Logging::message "cflags: %s\nldflags: %s\nlibs: %s\n\n",
                      cflags, ldflags, libs
     [cflags, ldflags, libs]
+  else
+    Logging::message "package configuration for %s is not found\n", pkg
+    nil
   end
 end
 
 def with_destdir(dir)
-  /^\$[\(\{]/ =~ dir ? dir : "$(DESTDIR)"+dir
+  dir = dir.sub($dest_prefix_pattern, '')
+  /\A\$[\(\{]/ =~ dir ? dir : "$(DESTDIR)"+dir
 end
 
 def winsep(s)
@@ -932,18 +1047,17 @@ topdir = #{($extmk ? CONFIG["topdir"] : $topdir).quote}
 hdrdir = #{$extmk ? CONFIG["hdrdir"].quote : '$(topdir)'}
 VPATH = #{vpath.join(CONFIG['PATH_SEPARATOR'])}
 }
-  drive = File::PATH_SEPARATOR == ';' ? /\A\w:/ : /\A/
-  if destdir = CONFIG["prefix"].scan(drive)[0] and !destdir.empty?
+  if destdir = CONFIG["prefix"][$dest_prefix_pattern, 1]
     mk << "\nDESTDIR = #{destdir}\n"
   end
   CONFIG.each do |key, var|
     next unless /prefix$/ =~ key
-    mk << "#{key} = #{with_destdir(var.sub(drive, ''))}\n"
+    mk << "#{key} = #{with_destdir(var)}\n"
   end
   CONFIG.each do |key, var|
     next if /^abs_/ =~ key
     next unless /^(?:src|top|hdr|(.*))dir$/ =~ key and $1
-    mk << "#{key} = #{with_destdir(var.sub(drive, ''))}\n"
+    mk << "#{key} = #{with_destdir(var)}\n"
   end
   if !$extmk and !$configure_args.has_key?('--ruby') and
       sep = config_string('BUILD_FILE_SEPARATOR')
@@ -951,6 +1065,7 @@ VPATH = #{vpath.join(CONFIG['PATH_SEPARATOR'])}
   else
     sep = ""
   end
+  extconf_h = $extconf_h ? "-DRUBY_EXTCONF_H=\\\"$(RUBY_EXTCONF_H)\\\" " : $defs.join(" ")<<" "
   mk << %{
 CC = #{CONFIG['CC']}
 LIBRUBY = #{CONFIG['LIBRUBY']}
@@ -958,8 +1073,10 @@ LIBRUBY_A = #{CONFIG['LIBRUBY_A']}
 LIBRUBYARG_SHARED = #$LIBRUBYARG_SHARED
 LIBRUBYARG_STATIC = #$LIBRUBYARG_STATIC
 
-CFLAGS   = #{CONFIG['CCDLFLAGS'] unless $static} #$CFLAGS #$ARCH_FLAG
-CPPFLAGS = -I. -I$(topdir) -I$(hdrdir) -I$(srcdir) #{$defs.join(" ")} #{$CPPFLAGS}
+RUBY_EXTCONF_H = #{$extconf_h}
+CFLAGS   = #{$static ? '' : CONFIG['CCDLFLAGS']} #$CFLAGS #$ARCH_FLAG
+INCFLAGS = -I. #$INCFLAGS
+CPPFLAGS = #{extconf_h}#{$CPPFLAGS}
 CXXFLAGS = $(CFLAGS) #{CONFIG['CXXFLAGS']}
 DLDFLAGS = #$LDFLAGS #$DLDFLAGS #$ARCH_FLAG
 LDSHARED = #{CONFIG['LDSHARED']}
@@ -982,7 +1099,7 @@ COPY = #{config_string('CP') || '@$(RUBY) -run -e cp -- -v'}
 
 #### End of system configuration section. ####
 
-preload = #{$preload.join(" ") if $preload}
+preload = #{$preload ? $preload.join(' ') : ''}
 }
   if $nmake == ?b
     mk.each do |x|
@@ -1006,9 +1123,26 @@ all install static install-so install-rb: Makefile
 RULES
 end
 
+# Generates the Makefile for your extension, passing along any options and
+# preprocessor constants that you may have generated through other methods.
+#
+# The +target+ name should correspond the name of the global function name
+# defined within your C extension, minus the 'Init_'.  For example, if your
+# C extension is defined as 'Init_foo', then your target would simply be 'foo'.
+#
+# If any '/' characters are present in the target name, only the last name
+# is interpreted as the target name, and the rest are considered toplevel
+# directory names, and the generated Makefile will be altered accordingly to
+# follow that directory structure.
+#
+# For example, if you pass 'test/foo' as a target name, your extension will
+# be installed under the 'test' directory.  This means that in order to
+# load the file within a Ruby program later, that directory structure will
+# have to be followed, e.g. "require 'test/foo'".
+#
 def create_makefile(target, srcprefix = nil)
   $target = target
-  libpath = $LIBPATH
+  libpath = $DEFLIBPATH|$LIBPATH
   message "creating Makefile\n"
   rm_f "conftest*"
   if CONFIG["DLEXT"] == $OBJEXT
@@ -1038,6 +1172,7 @@ def create_makefile(target, srcprefix = nil)
   elsif !(srcs = $srcs)
     srcs = $objs.collect {|obj| obj.sub(/\.o\z/, '.c')}
   end
+  $srcs = srcs
   for i in $objs
     i.sub!(/\.o\z/, ".#{$OBJEXT}")
   end
@@ -1060,6 +1195,7 @@ def create_makefile(target, srcprefix = nil)
       deffile = "$(TARGET)-$(arch).def"
     end
   end
+  origdef ||= ''
 
   libpath = libpathflag(libpath)
 
@@ -1067,8 +1203,8 @@ def create_makefile(target, srcprefix = nil)
   staticlib = target ? "$(TARGET).#$LIBEXT" : ""
   mfile = open("Makefile", "wb")
   mfile.print configuration(srcprefix)
-  mfile.print %{
-libpath = #{$LIBPATH.join(" ")}
+  mfile.print "
+libpath = #{($DEFLIBPATH|$LIBPATH).join(" ")}
 LIBPATH = #{libpath}
 DEFFILE = #{deffile}
 
@@ -1084,19 +1220,20 @@ SRCS = #{srcs.collect(&File.method(:basename)).join(' ')}
 OBJS = #{$objs}
 TARGET = #{target}
 DLLIB = #{dllib}
+EXTSTATIC = #{$static || ""}
 STATIC_LIB = #{staticlib unless $static.nil?}
-
-}
+#{!$extout && defined?($installed_list) ? "INSTALLED_LIST = #{$installed_list}\n" : ""}
+"
   install_dirs.each {|d| mfile.print("%-14s= %s\n" % d) if /^[[:upper:]]/ =~ d[0]}
   n = ($extout ? '$(RUBYARCHDIR)/' : '') + '$(TARGET).'
-  mfile.print %{
+  mfile.print "
 TARGET_SO     = #{($extout ? '$(RUBYARCHDIR)/' : '')}$(DLLIB)
 CLEANLIBS     = #{n}#{CONFIG['DLEXT']} #{n}il? #{n}tds #{n}map
 CLEANOBJS     = *.#{$OBJEXT} *.#{$LIBEXT} *.s[ol] *.pdb *.exp *.bak
 
-all:		#{target ? $extout ? "install" : "$(DLLIB)" : "Makefile"}
+all:		#{$extout ? "install" : target ? "$(DLLIB)" : "Makefile"}
 static:		$(STATIC_LIB)#{$extout ? " install-rb" : ""}
-}
+"
   mfile.print CLEANINGS
   dirs = []
   mfile.print "install: install-so install-rb\n\n"
@@ -1118,6 +1255,9 @@ static:		$(STATIC_LIB)#{$extout ? " install-rb" : ""}
         dir.gsub!(/(\$\{\w+)(\})/) {$1+sep+$2}
       end
       mfile.print "\t$(INSTALL_PROG) #{f} #{dir}\n"
+      if defined?($installed_list)
+	mfile.print "\t@echo #{dir}/#{File.basename(f)}>>$(INSTALLED_LIST)\n"
+      end
     end
   end
   mfile.print("install-rb: pre-install-rb install-rb-default\n")
@@ -1145,6 +1285,9 @@ static:		$(STATIC_LIB)#{$extout ? " install-rb" : ""}
 	  sep = ""
 	end
 	mfile.print("#{f} $(@D#{sep})\n")
+	if defined?($installed_list) and !$extout
+	  mfile.print("\t@echo #{dest}>>$(INSTALLED_LIST)\n")
+	end
       end
     end
   end
@@ -1179,10 +1322,11 @@ site-install-rb: install-rb
   end
 
   mfile.print "$(RUBYARCHDIR)/" if $extout
-  mfile.print "$(DLLIB): ", (makedef ? "$(DEFFILE) " : ""), "$(OBJS)\n\t"
-  mfile.print "@-$(RM) $@\n\t"
-  mfile.print "@-$(MAKEDIRS) $(@D)\n\t" if $extout
-  mfile.print LINK_SO, "\n\n"
+  mfile.print "$(DLLIB): ", (makedef ? "$(DEFFILE) " : ""), "$(OBJS)\n"
+  mfile.print "\t@-$(RM) $@\n"
+  mfile.print "\t@-$(MAKEDIRS) $(@D)\n" if $extout
+  link_so = LINK_SO.gsub(/^/, "\t")
+  mfile.print link_so, "\n\n"
   unless $static.nil?
     mfile.print "$(STATIC_LIB): $(OBJS)\n\t"
     mfile.print "$(AR) #{config_string('ARFLAGS') || 'cru '}$@ $(OBJS)"
@@ -1220,7 +1364,7 @@ site-install-rb: install-rb
 	  suffixes << m[1] << m[2]
 	  implicit = [[m[1], m[2]], [m.post_match]]
 	  next
-	elsif RULE_SUBST and /\A[$\w][^#]*:/ =~ line
+	elsif RULE_SUBST and /\A(?!\s*\w+\s*=)[$\w][^#]*:/ =~ line
 	  line.gsub!(%r"(\s)(?!\.)([^$(){}+=:\s\/\\,]+)(?=\s|\z)") {$1 + RULE_SUBST % $2}
 	end
 	depout << line
@@ -1253,6 +1397,7 @@ site-install-rb: install-rb
       headers.each {|h| h.sub!(/.*/) {|*m| RULE_SUBST % m}}
     end
     headers << $config_h if $config_h
+    headers << "$(RUBY_EXTCONF_H)" if $extconf_h
     mfile.print "$(OBJS): ", headers.join(' '), "\n"
   end
 
@@ -1266,11 +1411,12 @@ def init_mkmf(config = CONFIG)
   $arg_config = []
   $enable_shared = config['ENABLE_SHARED'] == 'yes'
   $defs = []
+  $extconf_h = nil
   $CFLAGS = with_config("cflags", arg_config("CFLAGS", config["CFLAGS"])).dup
   $ARCH_FLAG = with_config("arch_flag", arg_config("ARCH_FLAG", config["ARCH_FLAG"])).dup
   $CPPFLAGS = with_config("cppflags", arg_config("CPPFLAGS", config["CPPFLAGS"])).dup
-  $LDFLAGS = (with_config("ldflags") || "").dup
-  $INCFLAGS = "-I$(topdir)"
+  $LDFLAGS = with_config("ldflags", arg_config("LDFLAGS", config["LDFLAGS"])).dup
+  $INCFLAGS = "-I$(topdir) -I$(hdrdir) -I$(srcdir)"
   $DLDFLAGS = with_config("dldflags", arg_config("DLDFLAGS", config["DLDFLAGS"])).dup
   $LIBEXT = config['LIBEXT'].dup
   $OBJEXT = config["OBJEXT"].dup
@@ -1278,7 +1424,8 @@ def init_mkmf(config = CONFIG)
   $LIBRUBYARG = ""
   $LIBRUBYARG_STATIC = config['LIBRUBYARG_STATIC']
   $LIBRUBYARG_SHARED = config['LIBRUBYARG_SHARED']
-  $LIBPATH = $extmk ? ["$(topdir)"] : CROSS_COMPILING ? [] : ["$(libdir)"]
+  $DEFLIBPATH = $extmk ? ["$(topdir)"] : CROSS_COMPILING ? [] : ["$(libdir)"]
+  $LIBPATH = []
   $INSTALLFILES = nil
 
   $objs = nil
@@ -1310,7 +1457,7 @@ MESSAGE
 
 def mkmf_failed(path)
   unless $makefile_created or File.exist?("Makefile")
-    opts = $arg_config.collect {|t, n| "\t#{t}#{"=#{n}" if n}\n"}
+    opts = $arg_config.collect {|t, n| "\t#{t}#{n ? "=#{n}" : ""}\n"}
     abort "*** #{path} failed ***\n" + FailedMessage + opts.join
   end
 end
@@ -1348,23 +1495,22 @@ EXPORT_PREFIX = config_string('EXPORT_PREFIX') {|s| s.strip}
 
 hdr = []
 config_string('COMMON_MACROS') do |s|
-  Shellwords.shellwords(s).each do |s|
-    /(.*?)(?:=(.*))/ =~ s
-    hdr << "#define #$1 #$2"
+  Shellwords.shellwords(s).each do |w|
+    hdr << "#define " + w.split(/=/, 2).join(" ")
   end
 end
 config_string('COMMON_HEADERS') do |s|
   Shellwords.shellwords(s).each {|s| hdr << "#include <#{s}>"}
 end
-COMMON_HEADERS = (hdr.join("\n") unless hdr.empty?)
+COMMON_HEADERS = hdr.join("\n")
 COMMON_LIBS = config_string('COMMON_LIBS', &split) || []
 
 COMPILE_RULES = config_string('COMPILE_RULES', &split) || %w[.%s.%s:]
 RULE_SUBST = config_string('RULE_SUBST')
-COMPILE_C = config_string('COMPILE_C') || '$(CC) $(CFLAGS) $(CPPFLAGS) -c $<'
-COMPILE_CXX = config_string('COMPILE_CXX') || '$(CXX) $(CXXFLAGS) $(CPPFLAGS) -c $<'
+COMPILE_C = config_string('COMPILE_C') || '$(CC) $(INCFLAGS) $(CPPFLAGS) $(CFLAGS) -c $<'
+COMPILE_CXX = config_string('COMPILE_CXX') || '$(CXX) $(INCFLAGS) $(CPPFLAGS) $(CXXFLAGS) -c $<'
 TRY_LINK = config_string('TRY_LINK') ||
-  "$(CC) #{OUTFLAG}conftest $(INCFLAGS) -I$(hdrdir) $(CPPFLAGS) " \
+  "$(CC) #{OUTFLAG}conftest $(INCFLAGS) $(CPPFLAGS) " \
   "$(CFLAGS) $(src) $(LIBPATH) $(LDFLAGS) $(ARCH_FLAG) $(LOCAL_LIBS) $(LIBS)"
 LINK_SO = config_string('LINK_SO') ||
   if CONFIG["DLEXT"] == $OBJEXT
@@ -1377,13 +1523,13 @@ LIBPATHFLAG = config_string('LIBPATHFLAG') || ' -L"%s"'
 RPATHFLAG = config_string('RPATHFLAG') || ''
 LIBARG = config_string('LIBARG') || '-l%s'
 
-sep = File::ALT_SEPARATOR ? ":/=#{File::ALT_SEPARATOR}" : ''
+sep = config_string('BUILD_FILE_SEPARATOR') {|sep| ":/=#{sep}" if sep != "/"} || ""
 CLEANINGS = "
 clean:
 		@-$(RM) $(CLEANLIBS#{sep}) $(CLEANOBJS#{sep}) $(CLEANFILES#{sep})
 
 distclean:	clean
-		@-$(RM) Makefile extconf.h conftest.* mkmf.log
+		@-$(RM) Makefile $(RUBY_EXTCONF_H) conftest.* mkmf.log
 		@-$(RM) core ruby$(EXEEXT) *~ $(DISTCLEANFILES#{sep})
 
 realclean:	distclean
