@@ -10,8 +10,8 @@
 #
 # Main page:: http://www.germane-software.com/software/rexml
 # Author:: Sean Russell <serATgermaneHYPHENsoftwareDOTcom>
-# Version:: 3.1.3
-# Date:: 2005/224
+# Version:: 3.1.6
+# Date:: 2006/335
 # 
 # This API documentation can be downloaded from the REXML home page, or can
 # be accessed online[http://www.germane-software.com/software/rexml_doc]
@@ -20,7 +20,10 @@
 # or can be accessed 
 # online[http://www.germane-software.com/software/rexml/docs/tutorial.html]
 module REXML
-	Copyright = "Copyright © 2001, 2002, 2003, 2004 Sean Russell <ser@germane-software.com>"
-	Date = "2005/224"
-	Version = "3.1.3"
+	COPYRIGHT = "Copyright © 2001-2006 Sean Russell <ser@germane-software.com>"
+  DATE = "2006/335"
+  VERSION = "3.1.6"
+
+  Copyright = COPYRIGHT
+  Version = VERSION
 end
