@@ -268,7 +268,7 @@ Note: Inherited methods from class (({Object})) cannot be used as XML-RPC names,
 
 
 = History
-    $Id: client.rb,v 1.2.2.2 2005/09/20 08:51:02 matz Exp $
+    $Id: client.rb 11820 2007-02-23 03:47:59Z knu $
 
 =end
 
@@ -558,7 +558,7 @@ module XMLRPC
       expected = resp["Content-Length"] || "<unknown>"
       if data.nil? or data.size == 0 
         raise "Wrong size. Was #{data.size}, should be #{expected}" 
-      elsif expected.to_i != data.size and resp["Transfer-Encoding"].nil?
+      elsif expected != "<unknown>" and expected.to_i != data.size and resp["Transfer-Encoding"].nil?
         raise "Wrong size. Was #{data.size}, should be #{expected}"
       end
 
