@@ -2,8 +2,8 @@
 
   math.c -
 
-  $Author: matz $
-  $Date: 2006/07/27 16:14:03 $
+  $Author: shyouhei $
+  $Date: 2008-07-02 03:25:48 -0600 (Wed, 02 Jul 2008) $
   created at: Tue Jan 25 14:12:56 JST 1994
 
   Copyright (C) 1993-2003 Yukihiro Matsumoto
@@ -34,7 +34,7 @@ domain_check(x, msg)
 	if (isnan(x)) {
 #if defined(EDOM)
 	    errno = EDOM;
-#elif define(ERANGE)
+#elif defined(ERANGE)
 	    errno = ERANGE;
 #endif
 	    continue;
