@@ -2,8 +2,8 @@
 
   object.c -
 
-  $Author: akr $
-  $Date: 2005/12/12 03:36:49 $
+  $Author: shyouhei $
+  $Date: 2008-06-18 00:21:30 -0600 (Wed, 18 Jun 2008) $
   created at: Thu Jul 15 12:01:24 JST 1993
 
   Copyright (C) 1993-2003 Yukihiro Matsumoto
@@ -98,46 +98,6 @@ rb_obj_equal(obj1, obj2)
     return Qfalse;
 }
 
-
-/*
- *  Document-method: __id__
- *  Document-method: object_id
- *
- *  call-seq:
- *     obj.__id__       => fixnum
- *     obj.object_id    => fixnum
- *  
- *  Returns an integer identifier for <i>obj</i>. The same number will
- *  be returned on all calls to <code>id</code> for a given object, and
- *  no two active objects will share an id.
- *  <code>Object#object_id</code> is a different concept from the
- *  <code>:name</code> notation, which returns the symbol id of
- *  <code>name</code>. Replaces the deprecated <code>Object#id</code>.
- */
-
-
-
-/*
- *  call-seq:
- *     obj.hash    => fixnum
- *  
- *  Generates a <code>Fixnum</code> hash value for this object. This
- *  function must have the property that <code>a.eql?(b)</code> implies
- *  <code>a.hash == b.hash</code>. The hash value is used by class
- *  <code>Hash</code>. Any hash value that exceeds the capacity of a
- *  <code>Fixnum</code> will be truncated before being used.
- */
-
-VALUE
-rb_obj_id(obj)
-    VALUE obj;
-{
-    if (SPECIAL_CONST_P(obj)) {
-	return LONG2NUM((long)obj);
-    }
-    return (VALUE)((long)obj|FIXNUM_FLAG);
-}
-
 /*
  *  call-seq:
  *     obj.id    => fixnum
@@ -534,6 +494,34 @@ rb_obj_is_kind_of(obj, c)
 }
 
 
+/*
+ * Document-method: inherited
+ *
+ * call-seq:
+ *    inherited(subclass)
+ *
+ * Callback invoked whenever a subclass of the current class is created.
+ *
+ * Example:
+ *
+ *    class Foo
+ *       def self.inherited(subclass)
+ *          puts "New subclass: #{subclass}"
+ *       end
+ *    end
+ *
+ *    class Bar < Foo
+ *    end
+ *
+ *    class Baz < Bar
+ *    end
+ *
+ * produces:
+ *
+ *    New subclass: Bar
+ *    New subclass: Baz
+ */
+
 /*
  * Document-method: singleton_method_added
  *
@@ -645,7 +633,6 @@ rb_obj_dummy()
     return Qnil;
 }
 
-
 /*
  *  call-seq:
  *     obj.tainted?    => true or false
@@ -852,28 +839,6 @@ nil_inspect(obj)
     return rb_str_new2("nil");
 }
 
-#ifdef NIL_PLUS
-static VALUE
-nil_plus(x, y)
-    VALUE x, y;
-{
-    switch (TYPE(y)) {
-      case T_NIL:
-      case T_FIXNUM:
-      case T_FLOAT:
-      case T_BIGNUM:
-      case T_STRING:
-      case T_ARRAY:
-	return y;
-      default:
-	rb_raise(rb_eTypeError, "tried to add %s(%s) to nil",
-		 RSTRING(rb_inspect(y))->ptr,
-		 rb_obj_classname(y));
-    }
-    /* not reached */
-}
-#endif
-
 static VALUE
 main_to_s(obj)
     VALUE obj;
@@ -1646,13 +1611,9 @@ static ID
 str_to_id(str)
     VALUE str;
 {
-    if (!RSTRING(str)->ptr || RSTRING(str)->len == 0) {
-	rb_raise(rb_eArgError, "empty symbol string");
-    }
-    if (RSTRING(str)->len != strlen(RSTRING(str)->ptr)) {
-	rb_warn("Symbols should not contain NUL (\\0)");
-    }
-    return rb_intern(RSTRING(str)->ptr);
+    VALUE sym = rb_str_intern(str);
+
+    return SYM2ID(sym);
 }
 
 ID
@@ -1883,7 +1844,6 @@ rb_mod_const_defined(mod, name)
  *     k.methods.length   #=> 42
  */
 
-
 static VALUE
 rb_obj_methods(argc, argv, obj)
     int argc;
@@ -1984,10 +1944,12 @@ rb_obj_public_methods(argc, argv, obj)
 /*
  *  call-seq:
  *     obj.instance_variable_get(symbol)    => obj
- *  
- *  Returns the value of the given instance variable (or throws a
- *  <code>NameError</code> exception). The <code>@</code> part of the
- *  variable name should be included for regular instance variables
+ *
+ *  Returns the value of the given instance variable, or nil if the
+ *  instance variable is not set. The <code>@</code> part of the
+ *  variable name should be included for regular instance
+ *  variables. Throws a <code>NameError</code> exception if the
+ *  supplied symbol is not valid as an instance variable name.
  *     
  *     class Fred
  *       def initialize(p1, p2)
@@ -2011,7 +1973,6 @@ rb_obj_ivar_get(obj, iv)
     return rb_ivar_get(obj, id);
 }
 
-
 /*
  *  call-seq:
  *     obj.instance_variable_set(symbol, obj)    => obj
@@ -2044,6 +2005,36 @@ rb_obj_ivar_set(obj, iv, val)
     return rb_ivar_set(obj, id, val);
 }
 
+/*
+ *  call-seq:
+ *     obj.instance_variable_defined?(symbol)    => true or false
+ *
+ *  Returns <code>true</code> if the given instance variable is
+ *  defined in <i>obj</i>.
+ *
+ *     class Fred
+ *       def initialize(p1, p2)
+ *         @a, @b = p1, p2
+ *       end
+ *     end
+ *     fred = Fred.new('cat', 99)
+ *     fred.instance_variable_defined?(:@a)    #=> true
+ *     fred.instance_variable_defined?("@b")   #=> true
+ *     fred.instance_variable_defined?("@c")   #=> false
+ */
+
+static VALUE
+rb_obj_ivar_defined(obj, iv)
+    VALUE obj, iv;
+{
+    ID id = rb_to_id(iv);
+
+    if (!rb_is_instance_id(id)) {
+	rb_name_error(id, "`%s' is not allowed as an instance variable name", rb_id2name(id));
+    }
+    return rb_ivar_defined(obj, id);
+}
+
 /*
  *  call-seq:
  *     mod.class_variable_get(symbol)    => obj
@@ -2068,12 +2059,11 @@ rb_mod_cvar_get(obj, iv)
     ID id = rb_to_id(iv);
 
     if (!rb_is_class_id(id)) {
-	rb_name_error(id, "`%s' is not allowed as an class variable name", rb_id2name(id));
+	rb_name_error(id, "`%s' is not allowed as a class variable name", rb_id2name(id));
     }
     return rb_cvar_get(obj, id);
 }
 
-
 /*
  *  call-seq:
  *     obj.class_variable_set(symbol, obj)    => obj
@@ -2102,12 +2092,38 @@ rb_mod_cvar_set(obj, iv, val)
     ID id = rb_to_id(iv);
 
     if (!rb_is_class_id(id)) {
-	rb_name_error(id, "`%s' is not allowed as an class variable name", rb_id2name(id));
+	rb_name_error(id, "`%s' is not allowed as a class variable name", rb_id2name(id));
     }
     rb_cvar_set(obj, id, val, Qfalse);
     return val;
 }
 
+/*
+ *  call-seq:
+ *     obj.class_variable_defined?(symbol)    => true or false
+ *
+ *  Returns <code>true</code> if the given class variable is defined
+ *  in <i>obj</i>.
+ *
+ *     class Fred
+ *       @@foo = 99
+ *     end
+ *     Fred.class_variable_defined?(:@@foo)    #=> true
+ *     Fred.class_variable_defined?(:@@bar)    #=> false
+ */
+
+static VALUE
+rb_mod_cvar_defined(obj, iv)
+    VALUE obj, iv;
+{
+    ID id = rb_to_id(iv);
+
+    if (!rb_is_class_id(id)) {
+	rb_name_error(id, "`%s' is not allowed as a class variable name", rb_id2name(id));
+    }
+    return rb_cvar_defined(obj, id);
+}
+
 static VALUE
 convert_type(val, tname, method, raise)
     VALUE val;
@@ -2253,6 +2269,9 @@ rb_cstr_to_dbl(p, badcheck)
     const char *q;
     char *end;
     double d;
+    const char *ellipsis = "";
+    int w;
+#define OutOfRange() (((w = end - p) > 20) ? (w = 20, ellipsis = "...") : (ellipsis = ""))
 
     if (!p) return 0.0;
     q = p;
@@ -2262,9 +2281,11 @@ rb_cstr_to_dbl(p, badcheck)
     else {
 	while (ISSPACE(*p) || *p == '_') p++;
     }
+    errno = 0;
     d = strtod(p, &end);
     if (errno == ERANGE) {
-	rb_warn("Float %*s out of range", end-p, p);
+	OutOfRange();
+	rb_warn("Float %.*s%s out of range", w, p, ellipsis);
 	errno = 0;
     }
     if (p == end) {
@@ -2298,18 +2319,20 @@ rb_cstr_to_dbl(p, badcheck)
 	p = buf;
 	d = strtod(p, &end);
 	if (errno == ERANGE) {
-	    rb_warn("Float %*s out of range", end-p, p);
+	    OutOfRange();
+	    rb_warn("Float %.*s%s out of range", w, p, ellipsis);
 	    errno = 0;
 	}
 	if (badcheck) {
-	    if (p == end) goto bad;
+	    if (!end || p == end) goto bad;
 	    while (*end && ISSPACE(*end)) end++;
 	    if (*end) goto bad;
 	}
     }
     if (errno == ERANGE) {
 	errno = 0;
-	rb_raise(rb_eArgError, "Float %s out of range", q);
+	OutOfRange();
+	rb_raise(rb_eArgError, "Float %.*s%s out of range", w, q, ellipsis);
     }
     return d;
 }
@@ -2614,10 +2637,7 @@ Init_Object()
 
     rb_define_method(rb_mKernel, "eql?", rb_obj_equal, 1);
 
-    rb_define_method(rb_mKernel, "hash", rb_obj_id, 0);
     rb_define_method(rb_mKernel, "id", rb_obj_id_obsolete, 0);
-    rb_define_method(rb_mKernel, "__id__", rb_obj_id, 0);
-    rb_define_method(rb_mKernel, "object_id", rb_obj_id, 0);
     rb_define_method(rb_mKernel, "type", rb_obj_type, 0);
     rb_define_method(rb_mKernel, "class", rb_obj_class, 0);
 
@@ -2645,6 +2665,7 @@ Init_Object()
 		     rb_obj_instance_variables, 0); /* in variable.c */
     rb_define_method(rb_mKernel, "instance_variable_get", rb_obj_ivar_get, 1);
     rb_define_method(rb_mKernel, "instance_variable_set", rb_obj_ivar_set, 2);
+    rb_define_method(rb_mKernel, "instance_variable_defined?", rb_obj_ivar_defined, 1);
     rb_define_private_method(rb_mKernel, "remove_instance_variable",
 			     rb_obj_remove_instance_variable, 1); /* in variable.c */
 
@@ -2726,6 +2747,7 @@ Init_Object()
     rb_define_method(rb_cModule, "private_instance_methods", 
 		     rb_class_private_instance_methods, -1);   /* in class.c */
 
+    rb_define_method(rb_cModule, "class_variable_defined?", rb_mod_cvar_defined, 1);
     rb_define_method(rb_cModule, "constants", rb_mod_constants, 0); /* in variable.c */
     rb_define_method(rb_cModule, "const_get", rb_mod_const_get, 1);
     rb_define_method(rb_cModule, "const_set", rb_mod_const_set, 2);
