@@ -2,8 +2,8 @@
 
   process.c -
 
-  $Author: akr $
-  $Date: 2005/11/20 03:30:47 $
+  $Author: shyouhei $
+  $Date: 2008-06-29 03:33:11 -0600 (Sun, 29 Jun 2008) $
   created at: Tue Aug 10 14:30:50 JST 1993
 
   Copyright (C) 1993-2003 Yukihiro Matsumoto
@@ -42,7 +42,7 @@ struct timeval rb_time_interval _((VALUE));
 #ifdef HAVE_SYS_WAIT_H
 # include <sys/wait.h>
 #endif
-#ifdef HAVE_GETPRIORITY
+#ifdef HAVE_SYS_RESOURCE_H
 # include <sys/resource.h>
 #endif
 #include "st.h"
@@ -101,6 +101,13 @@ static VALUE S_Tms;
 #define BROKEN_SETREGID 1
 #endif
 
+#ifdef BROKEN_SETREUID
+#define setreuid ruby_setreuid
+#endif
+#ifdef BROKEN_SETREGID
+#define setregid ruby_setregid
+#endif
+
 #if defined(HAVE_44BSD_SETUID) || defined(__MacOS_X__)
 #if !defined(USE_SETREUID) && !defined(BROKEN_SETREUID)
 #define OBSOLETE_SETREUID 1
@@ -110,6 +117,9 @@ static VALUE S_Tms;
 #endif
 #endif
 
+#define preserving_errno(stmts) \
+	do {int saved_errno = errno; stmts; errno = saved_errno;} while (0)
+
 
 /*
  *  call-seq:
@@ -838,23 +848,22 @@ proc_waitall()
 }
 
 static VALUE
-detach_process_watcher(pid_p)
-    int *pid_p;
+detach_process_watcher(arg)
+    void *arg;
 {
-    int cpid, status;
+    int pid = (int)(VALUE)arg, status;
 
-    for (;;) {
-	cpid = rb_waitpid(*pid_p, &status, WNOHANG);
-	if (cpid != 0) return Qnil;
+    while (rb_waitpid(pid, &status, WNOHANG) == 0) {
 	rb_thread_sleep(1);
     }
+    return rb_last_status;
 }
 
 VALUE
 rb_detach_process(pid)
     int pid;
 {
-    return rb_thread_create(detach_process_watcher, (void*)&pid);
+    return rb_thread_create(detach_process_watcher, (void*)(VALUE)pid);
 }
 
 
@@ -873,6 +882,11 @@ rb_detach_process(pid)
  *  terminate.  <code>detach</code> only checks the status
  *  periodically (currently once each second).
  *
+ *  The waiting thread returns the exit status of the detached process
+ *  when it terminates, so you can use <code>Thread#join</code> to
+ *  know the result.  If specified _pid_ is not a valid child process
+ *  ID, the thread returns +nil+ immediately.
+ *
  *  In this first example, we don't reap the first child process, so
  *  it appears as a zombie in the process status display.
  *
@@ -981,8 +995,9 @@ proc_exec_v(argv, prog)
     }
 #endif /* MSDOS or __human68k__ or __EMX__ */
     before_exec();
+    rb_thread_cancel_timer();
     execv(prog, argv);
-    after_exec();
+    preserving_errno(after_exec());
     return -1;
 }
 
@@ -1051,7 +1066,7 @@ rb_proc_exec(str)
 #else
 	    before_exec();
 	    execl("/bin/sh", "sh", "-c", str, (char *)NULL);
-	    after_exec();
+	    preserving_errno(after_exec());
 #endif
 #endif
 	    return -1;
@@ -1060,8 +1075,8 @@ rb_proc_exec(str)
     a = argv = ALLOCA_N(char*, (s-str)/2+2);
     ss = ALLOCA_N(char, s-str+1);
     strcpy(ss, str);
-    if (*a++ = strtok(ss, " \t")) {
-	while (t = strtok(NULL, " \t")) {
+    if ((*a++ = strtok(ss, " \t")) != 0) {
+	while ((t = strtok(NULL, " \t")) != 0) {
 	    *a++ = t;
 	}
 	*a = NULL;
@@ -1136,13 +1151,13 @@ proc_spawn_n(argc, argv, prog)
     args = ALLOCA_N(char*, argc + 1);
     for (i = 0; i < argc; i++) {
 	SafeStringValue(argv[i]);
-	args[i] = RSTRING(argv[i])->ptr;
+	args[i] = StringValueCStr(argv[i]);
     }
     if (prog)
 	SafeStringValue(prog);
     args[i] = (char*) 0;
     if (args[0])
-	return proc_spawn_v(args, prog ? RSTRING(prog)->ptr : 0);
+	return proc_spawn_v(args, prog ? StringValueCStr(prog) : 0);
     return -1;
 }
 
@@ -1157,7 +1172,7 @@ proc_spawn(sv)
     int status;
 
     SafeStringValue(sv);
-    str = s = RSTRING(sv)->ptr;
+    str = s = StringValueCStr(sv);
     for (s = str; *s; s++) {
 	if (*s != ' ' && !ISALPHA(*s) && strchr("*?{}[]<>()~&|\\$;'`\"\n",*s)) {
 	    char *shell = dln_find_exe("sh", 0);
@@ -1180,6 +1195,53 @@ proc_spawn(sv)
 #endif
 #endif
 
+struct rb_exec_arg {
+    int argc;
+    VALUE *argv;
+    VALUE prog;
+};
+
+static void
+proc_prepare_args(e, argc, argv, prog)
+    struct rb_exec_arg *e;
+    int argc;
+    VALUE *argv;
+    VALUE prog;
+{
+    int i;
+
+    MEMZERO(e, struct rb_exec_arg, 1);
+    if (prog) {
+	SafeStringValue(prog);
+	StringValueCStr(prog);
+    }
+    for (i = 0; i < argc; i++) {
+	SafeStringValue(argv[i]);
+	StringValueCStr(argv[i]);
+    }
+    security(RSTRING(prog ? prog : argv[0])->ptr);
+    e->prog = prog;
+    e->argc = argc;
+    e->argv = argv;
+}
+
+static VALUE
+proc_exec_args(earg)
+    VALUE earg;
+{
+    struct rb_exec_arg *e = (struct rb_exec_arg *)earg;
+    int argc = e->argc;
+    VALUE *argv = e->argv;
+    VALUE prog = e->prog;
+
+    if (argc == 1 && prog == 0) {
+	return (VALUE)rb_proc_exec(RSTRING(argv[0])->ptr);
+    }
+    else {
+	return (VALUE)proc_exec_n(argc, argv, prog);
+    }
+}
+
 /*
  *  call-seq:
  *     exec(command [, arg, ...])
@@ -1212,8 +1274,10 @@ rb_f_exec(argc, argv)
 {
     VALUE prog = 0;
     VALUE tmp;
+    struct rb_exec_arg earg;
 
     if (argc == 0) {
+	rb_last_status = Qnil;
 	rb_raise(rb_eArgError, "wrong number of arguments");
     }
 
@@ -1226,15 +1290,8 @@ rb_f_exec(argc, argv)
 	argv[0] = RARRAY(tmp)->ptr[1];
 	SafeStringValue(prog);
     }
-    if (argc == 1 && prog == 0) {
-	VALUE cmd = argv[0];
-
-	SafeStringValue(cmd);
-	rb_proc_exec(RSTRING(cmd)->ptr);
-    }
-    else {
-	proc_exec_n(argc, argv, prog);
-    }
+    proc_prepare_args(&earg, argc, argv, prog);
+    proc_exec_args((VALUE)&earg);
     rb_sys_fail(RSTRING(argv[0])->ptr);
     return Qnil;		/* dummy */
 }
@@ -1458,7 +1515,7 @@ rb_f_system(argc, argv)
     if (argc == 1 && prog == 0) {
 #if defined(_WIN32)
 	SafeStringValue(argv[0]);
-	status = do_spawn(P_WAIT, RSTRING(argv[0])->ptr);
+	status = do_spawn(P_WAIT, StringValueCStr(argv[0]));
 #else
 	status = proc_spawn(argv[0]);
 #endif
@@ -1468,6 +1525,9 @@ rb_f_system(argc, argv)
     }
 #if !defined(_WIN32)
     last_status_set(status == -1 ? 127 : status, 0);
+#else
+    if (status == -1)
+	last_status_set(0x7f << 8, 0);
 #endif
 #elif defined(__VMS)
     VALUE cmd;
@@ -1486,12 +1546,13 @@ rb_f_system(argc, argv)
     cmd = rb_ary_join(rb_ary_new4(argc, argv), rb_str_new2(" "));
 
     SafeStringValue(cmd);
-    status = system(RSTRING(cmd)->ptr);
+    status = system(StringValueCStr(cmd));
     last_status_set((status & 0xff) << 8, 0);
 #else
     volatile VALUE prog = 0;
     int pid;
-    int i;
+    struct rb_exec_arg earg;
+    RETSIGTYPE (*chfunc)(int);
 
     fflush(stdout);
     fflush(stderr);
@@ -1507,37 +1568,28 @@ rb_f_system(argc, argv)
 	prog = RARRAY(argv[0])->ptr[0];
 	argv[0] = RARRAY(argv[0])->ptr[1];
     }
+    proc_prepare_args(&earg, argc, argv, prog);
 
-    if (prog) {
-	SafeStringValue(prog);
-    }
-    for (i = 0; i < argc; i++) {
-	SafeStringValue(argv[i]);
-    }
+    chfunc = signal(SIGCHLD, SIG_DFL);
   retry:
-    switch (pid = fork()) {
-      case 0:
-	if (argc == 1 && prog == 0) {
-	    rb_proc_exec(RSTRING(argv[0])->ptr);
-	}
-	else {
-	    proc_exec_n(argc, argv, prog);
-	}
+    pid = fork();
+    if (pid == 0) {
+	/* child process */
+	rb_thread_atfork();
+	rb_protect(proc_exec_args, (VALUE)&earg, NULL);
 	_exit(127);
-	break;			/* not reached */
-
-      case -1:
+    }
+    if (pid < 0) {
 	if (errno == EAGAIN) {
 	    rb_thread_sleep(1);
 	    goto retry;
 	}
-	rb_sys_fail(0);
-	break;
-
-      default:
+    }
+    else {
 	rb_syswait(pid);
     }
-
+    signal(SIGCHLD, chfunc);
+    if (pid < 0) rb_sys_fail(0);
     status = NUM2INT(rb_last_status);
 #endif
 
@@ -1549,12 +1601,11 @@ rb_f_system(argc, argv)
  *  call-seq:
  *     sleep([duration])    => fixnum
  *
- *  Suspends the current thread for _duration_ seconds (which may be
- *  any number, including a +Float+ with fractional seconds). Returns the actual
- *  number of seconds slept (rounded), which may be less than that asked
- *  for if the thread was interrupted by a +SIGALRM+, or if
- *  another thread calls <code>Thread#run</code>. Zero arguments
- *  causes +sleep+ to sleep forever.
+ *  Suspends the current thread for _duration_ seconds (which may be any number,
+ *  including a +Float+ with fractional seconds). Returns the actual number of
+ *  seconds slept (rounded), which may be less than that asked for if another
+ *  thread calls <code>Thread#run</code>. Zero arguments causes +sleep+ to sleep
+ *  forever.
  *
  *     Time.new    #=> Wed Apr 09 08:56:32 CDT 2003
  *     sleep 1.2   #=> 1
@@ -1598,6 +1649,10 @@ rb_f_sleep(argc, argv)
  *     Process.getpgrp      #=> 25527
  */
 
+#if defined(SIGCLD) && !defined(SIGCHLD)
+# define SIGCHLD SIGCLD
+#endif
+
 static VALUE
 proc_getpgrp()
 {
@@ -1819,6 +1874,115 @@ proc_setpriority(obj, which, who, prio)
 #endif
 }
 
+#if SIZEOF_RLIM_T == SIZEOF_INT
+# define RLIM2NUM(v) UINT2NUM(v)
+# define NUM2RLIM(v) NUM2UINT(v)
+#elif SIZEOF_RLIM_T == SIZEOF_LONG
+# define RLIM2NUM(v) ULONG2NUM(v)
+# define NUM2RLIM(v) NUM2ULONG(v)
+#elif SIZEOF_RLIM_T == SIZEOF_LONG_LONG
+# define RLIM2NUM(v) ULL2NUM(v)
+# define NUM2RLIM(v) NUM2ULL(v)
+#endif
+
+/*
+ *  call-seq:
+ *     Process.getrlimit(resource)   => [cur_limit, max_limit]
+ *
+ *  Gets the resource limit of the process.
+ *  _cur_limit_ means current (soft) limit and
+ *  _max_limit_ means maximum (hard) limit.
+ *
+ *  _resource_ indicates the kind of resource to limit:
+ *  such as <code>Process::RLIMIT_CORE</code>,
+ *  <code>Process::RLIMIT_CPU</code>, etc.
+ *  See Process.setrlimit for details.
+ *
+ *  _cur_limit_ and _max_limit_ may be <code>Process::RLIM_INFINITY</code>,
+ *  <code>Process::RLIM_SAVED_MAX</code> or
+ *  <code>Process::RLIM_SAVED_CUR</code>.
+ *  See Process.setrlimit and the system getrlimit(2) manual for details.
+ */
+
+static VALUE
+proc_getrlimit(VALUE obj, VALUE resource)
+{
+#if defined(HAVE_GETRLIMIT) && defined(RLIM2NUM)
+    struct rlimit rlim;
+
+    rb_secure(2);
+
+    if (getrlimit(NUM2INT(resource), &rlim) < 0) {
+        rb_sys_fail("getrlimit");
+    }
+    return rb_assoc_new(RLIM2NUM(rlim.rlim_cur), RLIM2NUM(rlim.rlim_max));
+#else
+    rb_notimplement();
+#endif
+}
+
+/*
+ *  call-seq:
+ *     Process.setrlimit(resource, cur_limit, max_limit)        => nil
+ *     Process.setrlimit(resource, cur_limit)                   => nil
+ *
+ *  Sets the resource limit of the process.
+ *  _cur_limit_ means current (soft) limit and
+ *  _max_limit_ means maximum (hard) limit.
+ *
+ *  If _max_limit_ is not given, _cur_limit_ is used.
+ *
+ *  _resource_ indicates the kind of resource to limit.
+ *  The list of resources are OS dependent.
+ *  Ruby may support following resources.
+ *
+ *  [Process::RLIMIT_CORE] core size (bytes) (SUSv3)
+ *  [Process::RLIMIT_CPU] CPU time (seconds) (SUSv3)
+ *  [Process::RLIMIT_DATA] data segment (bytes) (SUSv3)
+ *  [Process::RLIMIT_FSIZE] file size (bytes) (SUSv3)
+ *  [Process::RLIMIT_NOFILE] file descriptors (number) (SUSv3)
+ *  [Process::RLIMIT_STACK] stack size (bytes) (SUSv3)
+ *  [Process::RLIMIT_AS] total available memory (bytes) (SUSv3, NetBSD, FreeBSD, OpenBSD but 4.4BSD-Lite)
+ *  [Process::RLIMIT_MEMLOCK] total size for mlock(2) (bytes) (4.4BSD, GNU/Linux)
+ *  [Process::RLIMIT_NPROC] number of processes for the user (number) (4.4BSD, GNU/Linux)
+ *  [Process::RLIMIT_RSS] resident memory size (bytes) (4.2BSD, GNU/Linux)
+ *  [Process::RLIMIT_SBSIZE] all socket buffers (bytes) (NetBSD, FreeBSD)
+ *
+ *  Other <code>Process::RLIMIT_???</code> constants may be defined.
+ *
+ *  _cur_limit_ and _max_limit_ may be <code>Process::RLIM_INFINITY</code>,
+ *  which means that the resource is not limited.
+ *  They may be <code>Process::RLIM_SAVED_MAX</code> or
+ *  <code>Process::RLIM_SAVED_CUR</code> too.
+ *  See system setrlimit(2) manual for details.
+ *
+ */
+
+static VALUE
+proc_setrlimit(int argc, VALUE *argv, VALUE obj)
+{
+#if defined(HAVE_SETRLIMIT) && defined(NUM2RLIM)
+    VALUE resource, rlim_cur, rlim_max;
+    struct rlimit rlim;
+
+    rb_secure(2);
+
+    rb_scan_args(argc, argv, "21", &resource, &rlim_cur, &rlim_max);
+    if (rlim_max == Qnil)
+        rlim_max = rlim_cur;
+
+    rlim.rlim_cur = NUM2RLIM(rlim_cur);
+    rlim.rlim_max = NUM2RLIM(rlim_max);
+
+    if (setrlimit(NUM2INT(resource), &rlim) < 0) {
+        rb_sys_fail("setrlimit");
+    }
+    return Qnil;
+#else
+    rb_notimplement();
+#endif
+}
+
 static int under_uid_switch = 0;
 static void
 check_uid_switch()
@@ -2038,8 +2202,23 @@ proc_setuid(obj, id)
  *
  */
 
-static int SAVED_USER_ID;
+static int SAVED_USER_ID = -1;
 
+#ifdef BROKEN_SETREUID
+int
+setreuid(ruid, euid)
+    rb_uid_t ruid, euid;
+{
+    if (ruid != -1 && ruid != getuid()) {
+	if (euid == -1) euid = geteuid();
+	if (setuid(ruid) < 0) return -1;
+    }
+    if (euid != -1 && euid != geteuid()) {
+	if (seteuid(euid) < 0) return -1;
+    }
+    return 0;
+}
+#endif
 
 /*
  *  call-seq:
@@ -2383,7 +2562,7 @@ proc_setgid(obj, id)
 #elif defined HAVE_SETREGID
     if (setregid(gid, -1) < 0) rb_sys_fail(0);
 #elif defined HAVE_SETRGID
-    if (setrgid((GIDTYPE)gid) < 0) rb_sys_fail(0);
+    if (setrgid(gid) < 0) rb_sys_fail(0);
 #elif defined HAVE_SETGID
     {
 	if (getegid() == gid) {
@@ -2588,8 +2767,23 @@ proc_setmaxgroups(VALUE obj, VALUE val)
  *
  */
 
-static int SAVED_GROUP_ID;
+static int SAVED_GROUP_ID = -1;
 
+#ifdef BROKEN_SETREGID
+int
+setregid(rgid, egid)
+    rb_gid_t rgid, egid;
+{
+    if (rgid != -1 && rgid != getgid()) {
+	if (egid == -1) egid = getegid();
+	if (setgid(rgid) < 0) return -1;
+    }
+    if (egid != -1 && egid != getegid()) {
+	if (setegid(egid) < 0) return -1;
+    }
+    return 0;
+}
+#endif
 
 /*
  *  call-seq:
@@ -3324,6 +3518,10 @@ rb_proc_times(obj)
     VALUE obj;
 {
 #if defined(HAVE_TIMES) && !defined(__CHECKER__)
+    const double hertz =
+#ifdef HAVE__SC_CLK_TCK
+	(double)sysconf(_SC_CLK_TCK);
+#else
 #ifndef HZ
 # ifdef CLK_TCK
 #   define HZ CLK_TCK
@@ -3331,15 +3529,17 @@ rb_proc_times(obj)
 #   define HZ 60
 # endif
 #endif /* HZ */
+	HZ;
+#endif
     struct tms buf;
     volatile VALUE utime, stime, cutime, sctime;
 
     times(&buf);
     return rb_struct_new(S_Tms,
-			 utime = rb_float_new((double)buf.tms_utime / HZ),
-			 stime = rb_float_new((double)buf.tms_stime / HZ),
-			 cutime = rb_float_new((double)buf.tms_cutime / HZ),
-			 sctime = rb_float_new((double)buf.tms_cstime / HZ));
+			 utime = rb_float_new(buf.tms_utime / hertz),
+			 stime = rb_float_new(buf.tms_stime / hertz),
+			 cutime = rb_float_new(buf.tms_cutime / hertz),
+			 sctime = rb_float_new(buf.tms_cstime / hertz));
 #else
     rb_notimplement();
 #endif
@@ -3436,6 +3636,53 @@ Init_process()
     rb_define_const(rb_mProcess, "PRIO_USER", INT2FIX(PRIO_USER));
 #endif
 
+    rb_define_module_function(rb_mProcess, "getrlimit", proc_getrlimit, 1);
+    rb_define_module_function(rb_mProcess, "setrlimit", proc_setrlimit, -1);
+#ifdef RLIM2NUM
+#ifdef RLIM_INFINITY
+    rb_define_const(rb_mProcess, "RLIM_INFINITY", RLIM2NUM(RLIM_INFINITY));
+#endif
+#ifdef RLIM_SAVED_MAX
+    rb_define_const(rb_mProcess, "RLIM_SAVED_MAX", RLIM2NUM(RLIM_SAVED_MAX));
+#endif
+#ifdef RLIM_SAVED_CUR
+    rb_define_const(rb_mProcess, "RLIM_SAVED_CUR", RLIM2NUM(RLIM_SAVED_CUR));
+#endif
+#ifdef RLIMIT_CORE
+    rb_define_const(rb_mProcess, "RLIMIT_CORE", INT2FIX(RLIMIT_CORE));
+#endif
+#ifdef RLIMIT_CPU
+    rb_define_const(rb_mProcess, "RLIMIT_CPU", INT2FIX(RLIMIT_CPU));
+#endif
+#ifdef RLIMIT_DATA
+    rb_define_const(rb_mProcess, "RLIMIT_DATA", INT2FIX(RLIMIT_DATA));
+#endif
+#ifdef RLIMIT_FSIZE
+    rb_define_const(rb_mProcess, "RLIMIT_FSIZE", INT2FIX(RLIMIT_FSIZE));
+#endif
+#ifdef RLIMIT_NOFILE
+    rb_define_const(rb_mProcess, "RLIMIT_NOFILE", INT2FIX(RLIMIT_NOFILE));
+#endif
+#ifdef RLIMIT_STACK
+    rb_define_const(rb_mProcess, "RLIMIT_STACK", INT2FIX(RLIMIT_STACK));
+#endif
+#ifdef RLIMIT_AS
+    rb_define_const(rb_mProcess, "RLIMIT_AS", INT2FIX(RLIMIT_AS));
+#endif
+#ifdef RLIMIT_MEMLOCK
+    rb_define_const(rb_mProcess, "RLIMIT_MEMLOCK", INT2FIX(RLIMIT_MEMLOCK));
+#endif
+#ifdef RLIMIT_NPROC
+    rb_define_const(rb_mProcess, "RLIMIT_NPROC", INT2FIX(RLIMIT_NPROC));
+#endif
+#ifdef RLIMIT_RSS
+    rb_define_const(rb_mProcess, "RLIMIT_RSS", INT2FIX(RLIMIT_RSS));
+#endif
+#ifdef RLIMIT_SBSIZE
+    rb_define_const(rb_mProcess, "RLIMIT_SBSIZE", INT2FIX(RLIMIT_SBSIZE));
+#endif
+#endif
+
     rb_define_module_function(rb_mProcess, "uid", proc_getuid, 0);
     rb_define_module_function(rb_mProcess, "uid=", proc_setuid, 1);
     rb_define_module_function(rb_mProcess, "gid", proc_getgid, 0);
@@ -3470,8 +3717,8 @@ Init_process()
     rb_define_module_function(rb_mProcGID, "change_privilege", p_gid_change_privilege, 1);
     rb_define_module_function(rb_mProcUID, "grant_privilege", p_uid_grant_privilege, 1);
     rb_define_module_function(rb_mProcGID, "grant_privilege", p_gid_grant_privilege, 1);
-    rb_define_alias(rb_mProcUID, "eid=", "grant_privilege");
-    rb_define_alias(rb_mProcGID, "eid=", "grant_privilege");
+    rb_define_alias(rb_singleton_class(rb_mProcUID), "eid=", "grant_privilege");
+    rb_define_alias(rb_singleton_class(rb_mProcGID), "eid=", "grant_privilege");
     rb_define_module_function(rb_mProcUID, "re_exchange", p_uid_exchange, 0);
     rb_define_module_function(rb_mProcGID, "re_exchange", p_gid_exchange, 0);
     rb_define_module_function(rb_mProcUID, "re_exchangeable?", p_uid_exchangeable, 0);
