@@ -32,18 +32,17 @@ unless File.exist?(ruby)
   abort "#{ruby} is not found.\nTry `make' first, then `make test', please.\n"
 end
 
-libs = [abs_archdir, File.expand_path("lib", srcdir)]
+libs = [abs_archdir]
 if extout
   abs_extout = File.expand_path(extout)
-  libs << abs_extout << File.expand_path(RUBY_PLATFORM, abs_extout)
+  libs << File.expand_path("common", abs_extout) << File.expand_path(RUBY_PLATFORM, abs_extout)
 end
+libs << File.expand_path("lib", srcdir)
 config["bindir"] = abs_archdir
 ENV["RUBY"] = File.expand_path(ruby)
 ENV["PATH"] = [abs_archdir, ENV["PATH"]].compact.join(File::PATH_SEPARATOR)
 
-if e = ENV["RUBYLIB"]
-  libs |= e.split(File::PATH_SEPARATOR)
-end
+  libs << File.expand_path("ext", srcdir) << "-"
 ENV["RUBYLIB"] = $:.replace(libs).join(File::PATH_SEPARATOR)
 
 libruby_so = File.join(abs_archdir, config['LIBRUBY_SO'])
@@ -56,4 +55,7 @@ if File.file?(libruby_so)
   end
 end
 
-exec ruby, *ARGV
+cmd = [ruby]
+cmd << "-rpurelib.rb"
+cmd.concat(ARGV)
+exec(*cmd)
