@@ -2,8 +2,8 @@
 
   sprintf.c -
 
-  $Author: matz $
-  $Date: 2006/07/28 16:27:42 $
+  $Author: shyouhei $
+  $Date: 2008-06-19 17:12:46 -0600 (Thu, 19 Jun 2008) $
   created at: Fri Oct 15 10:39:26 JST 1993
 
   Copyright (C) 1993-2003 Yukihiro Matsumoto
@@ -18,6 +18,8 @@
 #include <math.h>
 
 #define BIT_DIGITS(N)   (((N)*146)/485 + 1)  /* log2(10) =~ 146/485 */
+#define BITSPERDIG (SIZEOF_BDIGITS*CHAR_BIT)
+#define EXTENDSIGN(n, l) (((~0 << (n)) >> (((n)*(l)) % BITSPERDIG)) & ~(~0 << (n)))
 
 static void fmt_setup _((char*,int,int,int,int));
 
@@ -36,7 +38,7 @@ remove_sign_bits(str, base)
 	}
     }
     else if (base == 8) {
-	if (*t == '3') t++;
+	*t |= EXTENDSIGN(3, strlen(t));
 	while (*t == '7') {
 	    t++;
 	}
@@ -247,7 +249,15 @@ rb_f_sprintf(argc, argv)
     int argc;
     VALUE *argv;
 {
+    return rb_str_format(argc - 1, argv + 1, GETNTHARG(0));
+}
+
+VALUE
+rb_str_format(argc, argv, fmt)
+    int argc;
+    VALUE *argv;
     VALUE fmt;
+{
     const char *p, *end;
     char *buf;
     int blen, bsiz;
@@ -276,7 +286,8 @@ rb_f_sprintf(argc, argv)
 	rb_raise(rb_eArgError, "flag after precision"); \
     }
 
-    fmt = GETNTHARG(0);
+    ++argc;
+    --argv;
     if (OBJ_TAINTED(fmt)) tainted = 1;
     StringValue(fmt);
     fmt = rb_str_new4(fmt);
@@ -428,23 +439,7 @@ rb_f_sprintf(argc, argv)
 			len = prec;
 		    }
 		}
-		{
-		    char *s, *send;
-		    long l;
-
-			s = RSTRING(str)->ptr;
-			send = s + RSTRING(str)->len;
-			l = 0;
-			while (s < send) {
-			    long n = mbclen(*s);
-			    if (l + n > len) {
-				len = l;
-				break;
-			    }
-			    l += n;
-			    s += n;
-			}
-		}
+		/* need to adjust multi-byte string pos */
 		if (flags&FWIDTH) {
 		    if (width > len) {
 			CHECK(width);
@@ -454,7 +449,7 @@ rb_f_sprintf(argc, argv)
 				buf[blen++] = ' ';
 			    }
 			}
-			memcpy(&buf[blen], RSTRING(str)->ptr, len);
+			memcpy(&buf[blen], RSTRING_PTR(str), len);
 			blen += len;
 			if (flags&FMINUS) {
 			    while (width--) {
@@ -485,7 +480,7 @@ rb_f_sprintf(argc, argv)
 		long v = 0;
 		int base, bignum = 0;
 		int len, pos;
-		VALUE tmp;
+		volatile VALUE tmp;
                 volatile VALUE tmp1;
 
 		switch (*p) {
@@ -636,7 +631,7 @@ rb_f_sprintf(argc, argv)
 		    val = rb_big_clone(val);
 		    rb_big_2comp(val);
 		}
-		tmp1 = tmp = rb_big2str(val, base);
+		tmp1 = tmp = rb_big2str0(val, base, RBIGNUM(val)->sign);
 		s = RSTRING(tmp)->ptr;
 		if (*s == '-') {
 		    if (base == 10) {
@@ -743,6 +738,8 @@ rb_f_sprintf(argc, argv)
 		    need = strlen(expr);
 		    if ((!isnan(fval) && fval < 0.0) || (flags & FPLUS))
 			need++;
+		    else if (flags & FSPACE)
+			need++;
 		    if ((flags & FWIDTH) && need < width)
 			need = width;
 
