@@ -2,8 +2,8 @@
 
   struct.c -
 
-  $Author: matz $
-  $Date: 2006/02/02 07:18:10 $
+  $Author: shyouhei $
+  $Date: 2007-02-13 08:01:19 +0900 (Tue, 13 Feb 2007) $
   created at: Tue Mar 22 18:44:30 JST 1995
 
   Copyright (C) 1993-2003 Yukihiro Matsumoto
@@ -252,7 +252,7 @@ rb_struct_define(name, va_alist)
     ary = rb_ary_new();
 
     va_init_list(ar, name);
-    while ((mem = va_arg(ar, char*)) != 0) {
+    while (mem = va_arg(ar, char*)) {
 	ID slot = rb_intern(mem);
 	rb_ary_push(ary, ID2SYM(slot));
     }
@@ -557,9 +557,8 @@ rb_struct_init_copy(copy, s)
     if (!rb_obj_is_instance_of(s, rb_obj_class(copy))) {
 	rb_raise(rb_eTypeError, "wrong argument class");
     }
-    if (RSTRUCT(copy)->len != RSTRUCT(s)->len) {
-	rb_raise(rb_eTypeError, "struct size mismatch");
-    }
+    RSTRUCT(copy)->ptr = ALLOC_N(VALUE, RSTRUCT(s)->len);
+    RSTRUCT(copy)->len = RSTRUCT(s)->len;
     MEMCPY(RSTRUCT(copy)->ptr, RSTRUCT(s)->ptr, VALUE, RSTRUCT(copy)->len);
 
     return copy;
