@@ -1,6 +1,6 @@
 # -*- mode: ruby; ruby-indent-level: 4; tab-width: 4 -*-
 #												vim:sw=4:ts=4
-# $Id: test_yaml.rb,v 1.5.2.2 2005/09/17 17:17:07 why Exp $
+# $Id: test_yaml.rb 17246 2008-06-15 13:12:40Z shyouhei $
 #
 require 'test/unit'
 require 'yaml'
@@ -1225,6 +1225,61 @@ EOY
         inspect_str = "[[...], [...]]"
         assert_equal( inspect_str, YAML::load( a.to_yaml ).inspect )
     end
+
+    #
+    # Test Symbol cycle
+    #
+    def test_symbol_cycle
+      #
+      # From Aaron Schrab [ruby-Bugs:2535]
+      #
+      assert_cycle(:"^foo")
+    end
+
+    #
+    # Test Numeric cycle
+    #
+    class NumericTest < Numeric
+      def initialize(value)
+        @value = value
+      end
+      def ==(other)
+        @value == other.instance_eval{ @value }
+      end
+    end
+    def test_numeric_cycle
+      assert_cycle(1) # Fixnum
+      assert_cycle(111111111111111111111111111111111) # Bignum
+      assert_cycle(NumericTest.new(3)) # Subclass of Numeric
+    end
+
+    #
+    # Test empty map/seq in map cycle
+    #
+    def test_empty_map_key
+      #
+      # empty seq as key
+      #
+      o = YAML.load({[]=>""}.to_yaml)
+      assert_equal(Hash, o.class)
+      assert_equal([[]], o.keys)
+
+      #
+      # empty map as key
+      #
+      o = YAML.load({{}=>""}.to_yaml)
+      assert_equal(Hash, o.class)
+      assert_equal([{}], o.keys)
+    end
+
+    #
+    # contributed by riley lynch [ruby-Bugs-8548]
+    #
+    def test_object_id_collision
+      omap = YAML::Omap.new
+      1000.times { |i| omap["key_#{i}"] = { "value" => i } }
+      raise "id collision in ordered map" if omap.to_yaml =~ /id\d+/
+    end
 end
 
 if $0 == __FILE__
