@@ -2,8 +2,8 @@
 
   time.c -
 
-  $Author: nobu $
-  $Date: 2006/07/27 13:23:18 $
+  $Author: shyouhei $
+  $Date: 2007-08-22 10:28:12 +0900 (Wed, 22 Aug 2007) $
   created at: Tue Dec 28 14:31:59 JST 1993
 
   Copyright (C) 1993-2003 Yukihiro Matsumoto
@@ -1066,7 +1066,7 @@ static VALUE
 time_dup(time)
     VALUE time;
 {
-    VALUE dup = time_s_alloc(rb_cTime);
+    VALUE dup = time_s_alloc(CLASS_OF(time));
     time_init_copy(dup, time);
     return dup;
 }
@@ -1865,7 +1865,7 @@ time_strftime(time, format)
 	while (p < pe) {
 	    len = rb_strftime(&buf, p, &tobj->tm);
 	    rb_str_cat(str, buf, len);
-	    p += strlen(p) + 1;
+	    p += strlen(p);
 	    if (buf != buffer) {
 		free(buf);
 		buf = buffer;
