@@ -2,8 +2,8 @@
 
   variable.c -
 
-  $Author: matz $
-  $Date: 2006/07/20 07:04:17 $
+  $Author: shyouhei $
+  $Date: 2009-02-01 19:34:12 -0700 (Sun, 01 Feb 2009) $
   created at: Tue Apr 19 23:55:15 JST 1994
 
   Copyright (C) 1993-2003 Yukihiro Matsumoto
@@ -642,6 +642,7 @@ rb_f_untrace_var(argc, argv)
     struct global_entry *entry;
     struct trace_var *trace;
 
+    rb_secure(4);
     rb_scan_args(argc, argv, "11", &var, &cmd);
     id = rb_to_id(var);
     if (!st_lookup(rb_global_tbl, id, (st_data_t *)&entry)) {
@@ -1324,12 +1325,8 @@ autoload_delete(mod, id)
 	st_delete(tbl, (st_data_t*)&id, &load);
 
 	if (tbl->num_entries == 0) {
-	    DATA_PTR(val) = 0;
-	    st_free_table(tbl);
 	    id = autoload;
-	    if (st_delete(RCLASS(mod)->iv_tbl, (st_data_t*)&id, &val)) {
-		rb_gc_force_recycle(val);
-	    }
+	    st_delete(RCLASS(mod)->iv_tbl, (st_data_t*)&id, &val);
 	}
     }
 
@@ -1375,12 +1372,8 @@ autoload_file(mod, id)
     /* already loaded but not defined */
     st_delete(tbl, (st_data_t*)&id, 0);
     if (!tbl->num_entries) {
-	DATA_PTR(val) = 0;
-	st_free_table(tbl);
 	id = autoload;
-	if (st_delete(RCLASS(mod)->iv_tbl, (st_data_t*)&id, &val)) {
-	    rb_gc_force_recycle(val);
-	}
+	st_delete(RCLASS(mod)->iv_tbl, (st_data_t*)&id, &val);
     }
     return Qnil;
 }
@@ -1691,7 +1684,7 @@ rb_define_const(klass, name, val)
     ID id = rb_intern(name);
 
     if (!rb_is_const_id(id)) {
-	rb_warn("rb_define_const: invalide name `%s' for constant", name);
+	rb_warn("rb_define_const: invalid name `%s' for constant", name);
     }
     if (klass == rb_cObject) {
 	rb_secure(4);
