@@ -82,8 +82,6 @@
 
 #define TO_SOCKET(x)	_get_osfhandle(x)
 
-bool NtSyncProcess = TRUE;
-
 static struct ChildRecord *CreateChild(const char *, const char *, SECURITY_ATTRIBUTES *, HANDLE, HANDLE, HANDLE);
 static bool has_redirection(const char *);
 static void StartSockets ();
@@ -503,26 +501,6 @@ FindFreeChildSlot(void)
 }
 
 
-int
-SafeFree(char **vec, int vecc)
-{
-    //   vec
-    //   |
-    //   V       ^---------------------V
-    //   +---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+
-    //   |   |       | ....  |  NULL |   | ..... |\0 |   | ..... |\0 |...
-    //   +---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+
-    //   |-  elements+1             -| ^ 1st element   ^ 2nd element
-
-	char *p;
-
-	p = (char *)vec;
-	free(p);
-
-	return 0;
-}
-
-
 /*
   ruby -lne 'BEGIN{$cmds = Hash.new(0); $mask = 1}'
    -e '$cmds[$_.downcase] |= $mask' -e '$mask <<= 1 if ARGF.eof'
@@ -1119,7 +1097,7 @@ cmdglob(NtCmdLineElement *patt, NtCmdLineElement **tail)
     for (p = buf; *p; p = CharNext(p))
 	if (*p == '\\')
 	    *p = '/';
-    status = ruby_globi(buf, 0, insert, (VALUE)&tail);
+    status = ruby_glob(buf, 0, insert, (VALUE)&tail);
     if (buf != buffer)
 	free(buf);
 
@@ -1433,6 +1411,13 @@ rb_w32_cmdvector(const char *cmd, char ***vec)
 // return the pointer to the current file name. 
 //
 
+#define GetBit(bits, i) ((bits)[(i) / CHAR_BIT] &  (1 << (i) % CHAR_BIT))
+#define SetBit(bits, i) ((bits)[(i) / CHAR_BIT] |= (1 << (i) % CHAR_BIT))
+
+#define BitOfIsDir(n) ((n) * 2)
+#define BitOfIsRep(n) ((n) * 2 + 1)
+#define DIRENT_PER_CHAR (CHAR_BIT / 2)
+
 DIR *
 rb_w32_opendir(const char *filename)
 {
@@ -1494,7 +1479,7 @@ rb_w32_opendir(const char *filename)
     //
 
     idx = strlen(fd.cFileName)+1;
-    if (!(p->start = (char *)malloc(idx))) {
+    if (!(p->start = (char *)malloc(idx)) || !(p->bits = (char *)malloc(1))) {
       error:
 	rb_w32_closedir(p);
 	FindClose(fh);
@@ -1502,6 +1487,11 @@ rb_w32_opendir(const char *filename)
 	return NULL;
     }
     strcpy(p->start, fd.cFileName);
+    p->bits[0] = 0;
+    if (fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
+	SetBit(p->bits, BitOfIsDir(0));
+    if (fd.dwFileAttributes & FILE_ATTRIBUTE_REPARSE_POINT)
+	SetBit(p->bits, BitOfIsRep(0));
     p->nfiles++;
 
     //
@@ -1513,21 +1503,34 @@ rb_w32_opendir(const char *filename)
     while (FindNextFile(fh, &fd)) {
 	char *newpath;
 
-	len = strlen(fd.cFileName);
+	len = strlen(fd.cFileName) + 1;
 
 	//
 	// bump the string table size by enough for the
 	// new name and it's null terminator 
 	//
 
-	newpath = (char *)realloc(p->start, idx+len+1);
+	newpath = (char *)realloc(p->start, idx + len);
 	if (newpath == NULL) {
 	    goto error;
 	}
 	p->start = newpath;
 	strcpy(&p->start[idx], fd.cFileName);
+
+	if (p->nfiles % DIRENT_PER_CHAR == 0) {
+	    char *tmp = realloc(p->bits, p->nfiles / DIRENT_PER_CHAR + 1);
+	    if (!tmp)
+		goto error;
+	    p->bits = tmp;
+	    p->bits[p->nfiles / DIRENT_PER_CHAR] = 0;
+	}
+	if (fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
+	    SetBit(p->bits, BitOfIsDir(p->nfiles));
+	if (fd.dwFileAttributes & FILE_ATTRIBUTE_REPARSE_POINT)
+	    SetBit(p->bits, BitOfIsRep(p->nfiles));
+
 	p->nfiles++;
-	idx += len+1;
+	idx += len;
     }
     FindClose(fh);
     p->size = idx;
@@ -1543,6 +1546,7 @@ static void
 move_to_next_entry(DIR *dirp)
 {
     if (dirp->curr) {
+	dirp->loc++;
 	dirp->curr += strlen(dirp->curr) + 1;
 	if (dirp->curr >= (dirp->start + dirp->size)) {
 	    dirp->curr = NULL;
@@ -1574,6 +1578,12 @@ rb_w32_readdir(DIR *dirp)
 	//
 	dirp->dirstr.d_ino = dummy++;
 
+	//
+	// Attributes
+	//
+	dirp->dirstr.d_isdir = GetBit(dirp->bits, BitOfIsDir(dirp->loc));
+	dirp->dirstr.d_isrep = GetBit(dirp->bits, BitOfIsRep(dirp->loc));
+
 	//
 	// Now set up for the next call to readdir
 	//
@@ -1636,6 +1646,7 @@ void
 rb_w32_closedir(DIR *dirp)
 {
     free(dirp->start);
+    free(dirp->bits);
     free(dirp);
 }
 
@@ -2692,7 +2703,7 @@ poll_child_status(struct ChildRecord *child, int *stat_loc)
 }
 
 rb_pid_t
-waitpid (rb_pid_t pid, int *stat_loc, int options)
+waitpid(rb_pid_t pid, int *stat_loc, int options)
 {
     DWORD timeout;
 
@@ -3347,6 +3358,12 @@ setup_call(CONTEXT* ctx, struct handler_arg_t *harg)
 #endif
 }
 
+void
+rb_w32_interrupted(void)
+{
+    SetSignal(interrupted_event);
+}
+
 int
 rb_w32_main_context(int arg, void (*handler)(int))
 {
@@ -3358,7 +3375,7 @@ rb_w32_main_context(int arg, void (*handler)(int))
 
     if (GetCurrentThreadId() == main_thread.id) return FALSE;
 
-    SetSignal(interrupted_event);
+    rb_w32_interrupted();
 
     RUBY_CRITICAL({		/* the main thread must be in user state */
 	CONTEXT ctx;
@@ -3723,11 +3740,17 @@ rb_w32_utime(const char *path, struct utimbuf *times)
 	return -1;
     }
 
-    if (unixtime_to_filetime(times->actime, &atime)) {
-	return -1;
+    if (times) {
+	if (unixtime_to_filetime(times->actime, &atime)) {
+	    return -1;
+	}
+	if (unixtime_to_filetime(times->modtime, &mtime)) {
+	    return -1;
+	}
     }
-    if (unixtime_to_filetime(times->modtime, &mtime)) {
-	return -1;
+    else {
+	GetSystemTimeAsFileTime(&atime);
+	mtime = atime;
     }
 
     RUBY_CRITICAL({
